import urllib


class ActionBatchNetworks(object):
    def __init__(self):
        super(ActionBatchNetworks, self).__init__()
        


    def updateNetwork(self, networkId: str, **kwargs):
        """
        **Update a network**
        https://developer.cisco.com/meraki/api-v1/#!update-network

        - networkId (string): Network ID
        - name (string): The name of the network
        - timeZone (string): The timezone of the network. For a list of allowed timezones, please see the 'TZ' column in the table in <a target='_blank' href='https://en.wikipedia.org/wiki/List_of_tz_database_time_zones'>this article.</a>
        - tags (array): A list of tags to be applied to the network
        - enrollmentString (string): A unique identifier which can be used for device enrollment or easy access through the Meraki SM Registration page or the Self Service Portal. Please note that changing this field may cause existing bookmarks to break.
        - notes (string): Add any notes or additional information about this network here.
        """

        kwargs.update(locals())

        metadata = {
            'tags': ['networks', 'configure'],
            'operation': 'updateNetwork'
        }
        resource = f'/networks/{networkId}'

        body_params = ['name', 'timeZone', 'tags', 'enrollmentString', 'notes', ]
        payload = {k.strip(): v for k, v in kwargs.items() if k.strip() in body_params}
        action = {
            "resource": resource,
            "operation": "update",
            "body": payload
        }
        return action
        





    def deleteNetwork(self, networkId: str):
        """
        **Delete a network**
        https://developer.cisco.com/meraki/api-v1/#!delete-network

        - networkId (string): Network ID
        """

        metadata = {
            'tags': ['networks', 'configure'],
            'operation': 'deleteNetwork'
        }
        resource = f'/networks/{networkId}'

        action = {
            "resource": resource,
            "operation": "destroy",
        }
        return action
        





    def bindNetwork(self, networkId: str, configTemplateId: str, **kwargs):
        """
        **Bind a network to a template.**
        https://developer.cisco.com/meraki/api-v1/#!bind-network

        - networkId (string): Network ID
        - configTemplateId (string): The ID of the template to which the network should be bound.
        - autoBind (boolean): Optional boolean indicating whether the network's switches should automatically bind to profiles of the same model. Defaults to false if left unspecified. This option only affects switch networks and switch templates. Auto-bind is not valid unless the switch template has at least one profile and has at most one profile per switch model.
        """

        kwargs.update(locals())

        metadata = {
            'tags': ['networks', 'configure'],
            'operation': 'bindNetwork'
        }
        resource = f'/networks/{networkId}/bind'

        body_params = ['configTemplateId', 'autoBind', ]
        payload = {k.strip(): v for k, v in kwargs.items() if k.strip() in body_params}
        action = {
            "resource": resource,
            "operation": "bind",
            "body": payload
        }
        return action
        





    def provisionNetworkClients(self, networkId: str, clients: list, devicePolicy: str, **kwargs):
        """
        **Provisions a client with a name and policy. Clients can be provisioned before they associate to the network.**
        https://developer.cisco.com/meraki/api-v1/#!provision-network-clients

        - networkId (string): Network ID
        - clients (array): The array of clients to provision
        - devicePolicy (string): The policy to apply to the specified client. Can be 'Group policy', 'Allowed', 'Blocked', 'Per connection' or 'Normal'. Required.
        - groupPolicyId (string): The ID of the desired group policy to apply to the client. Required if 'devicePolicy' is set to "Group policy". Otherwise this is ignored.
        - policiesBySecurityAppliance (object): An object, describing what the policy-connection association is for the security appliance. (Only relevant if the security appliance is actually within the network)
        - policiesBySsid (object): An object, describing the policy-connection associations for each active SSID within the network. Keys should be the number of enabled SSIDs, mapping to an object describing the client's policy
        """

        kwargs.update(locals())

        if 'devicePolicy' in kwargs:
            options = ['Allowed', 'Blocked', 'Group policy', 'Normal', 'Per connection']
            assert kwargs['devicePolicy'] in options, f'''"devicePolicy" cannot be "{kwargs['devicePolicy']}", & must be set to one of: {options}'''

        metadata = {
            'tags': ['networks', 'configure', 'clients'],
            'operation': 'provisionNetworkClients'
        }
        resource = f'/networks/{networkId}/clients/provision'

        body_params = ['clients', 'devicePolicy', 'groupPolicyId', 'policiesBySecurityAppliance', 'policiesBySsid', ]
        payload = {k.strip(): v for k, v in kwargs.items() if k.strip() in body_params}
        action = {
            "resource": resource,
            "operation": "provision",
            "body": payload
        }
        return action
        





    def claimNetworkDevices(self, networkId: str, serials: list, **kwargs):
        """
        **Claim devices into a network. (Note: for recently claimed devices, it may take a few minutes for API requests against that device to succeed). This operation can be used up to ten times within a single five minute window.**
        https://developer.cisco.com/meraki/api-v1/#!claim-network-devices

        - networkId (string): Network ID
        - serials (array): A list of serials of devices to claim
        - addAtomically (boolean): Whether to claim devices atomically. If true, all devices will be claimed or none will be claimed. Default is true.
        - detailsByDevice (array): Optional details for claimed devices (currently only used for Catalyst devices)
        """

        kwargs.update(locals())

        metadata = {
            'tags': ['networks', 'configure', 'devices'],
            'operation': 'claimNetworkDevices'
        }
        resource = f'/networks/{networkId}/devices/claim'

        body_params = ['serials', 'detailsByDevice', ]
        payload = {k.strip(): v for k, v in kwargs.items() if k.strip() in body_params}
        action = {
            "resource": resource,
            "operation": "claim",
            "body": payload
        }
        return action
        





    def vmxNetworkDevicesClaim(self, networkId: str, size: str):
        """
        **Claim a vMX into a network**
        https://developer.cisco.com/meraki/api-v1/#!vmx-network-devices-claim

        - networkId (string): Network ID
        - size (string): The size of the vMX you claim. It can be one of: small, medium, large, xlarge, 100
        """

        kwargs = locals()

        if 'size' in kwargs:
            options = ['100', 'large', 'medium', 'small', 'xlarge']
            assert kwargs['size'] in options, f'''"size" cannot be "{kwargs['size']}", & must be set to one of: {options}'''

        metadata = {
            'tags': ['networks', 'configure', 'devices', 'claim'],
            'operation': 'vmxNetworkDevicesClaim'
        }
        resource = f'/networks/{networkId}/devices/claim/vmx'

        body_params = ['size', ]
        payload = {k.strip(): v for k, v in kwargs.items() if k.strip() in body_params}
        action = {
            "resource": resource,
            "operation": "claim",
            "body": payload
        }
        return action
        





    def removeNetworkDevices(self, networkId: str, serial: str):
        """
        **Remove a single device**
        https://developer.cisco.com/meraki/api-v1/#!remove-network-devices

        - networkId (string): Network ID
        - serial (string): The serial of a device
        """

        kwargs = locals()

        metadata = {
            'tags': ['networks', 'configure', 'devices'],
            'operation': 'removeNetworkDevices'
        }
        resource = f'/networks/{networkId}/devices/remove'

        body_params = ['serial', ]
        payload = {k.strip(): v for k, v in kwargs.items() if k.strip() in body_params}
        action = {
            "resource": resource,
            "operation": "remove",
            "body": payload
        }
        return action
        





    def updateNetworkFirmwareUpgrades(self, networkId: str, **kwargs):
        """
        **Update firmware upgrade information for a network**
        https://developer.cisco.com/meraki/api-v1/#!update-network-firmware-upgrades

        - networkId (string): Network ID
        - upgradeWindow (object): Upgrade window for devices in network
        - timezone (string): The timezone for the network
        - products (object): Contains information about the network to update
        """

        kwargs.update(locals())

        metadata = {
            'tags': ['networks', 'configure', 'firmwareUpgrades'],
            'operation': 'updateNetworkFirmwareUpgrades'
        }
        resource = f'/networks/{networkId}/firmwareUpgrades'

        body_params = ['upgradeWindow', 'timezone', 'products', ]
        payload = {k.strip(): v for k, v in kwargs.items() if k.strip() in body_params}
        action = {
            "resource": resource,
            "operation": "update",
            "body": payload
        }
        return action
        





    def createNetworkFirmwareUpgradesRollback(self, networkId: str, reasons: list, **kwargs):
        """
        **Rollback a Firmware Upgrade For A Network**
        https://developer.cisco.com/meraki/api-v1/#!create-network-firmware-upgrades-rollback

        - networkId (string): Network ID
        - reasons (array): Reasons for the rollback
        - product (string): Product type to rollback (if the network is a combined network)
        - time (string): Scheduled time for the rollback
        - toVersion (object): Version to downgrade to (if the network has firmware flexibility)
        """

        kwargs.update(locals())

        if 'product' in kwargs:
            options = ['appliance', 'camera', 'cellularGateway', 'secureConnect', 'switch', 'switchCatalyst', 'wireless', 'wirelessController']
            assert kwargs['product'] in options, f'''"product" cannot be "{kwargs['product']}", & must be set to one of: {options}'''

        metadata = {
            'tags': ['networks', 'configure', 'firmwareUpgrades', 'rollbacks'],
            'operation': 'createNetworkFirmwareUpgradesRollback'
        }
        resource = f'/networks/{networkId}/firmwareUpgrades/rollbacks'

        body_params = ['product', 'time', 'reasons', 'toVersion', ]
        payload = {k.strip(): v for k, v in kwargs.items() if k.strip() in body_params}
        action = {
            "resource": resource,
            "operation": "create",
            "body": payload
        }
        return action
        





    def createNetworkFirmwareUpgradesStagedGroup(self, networkId: str, name: str, isDefault: bool, **kwargs):
        """
        **Create a Staged Upgrade Group for a network**
        https://developer.cisco.com/meraki/api-v1/#!create-network-firmware-upgrades-staged-group

        - networkId (string): Network ID
        - name (string): Name of the Staged Upgrade Group. Length must be 1 to 255 characters
        - isDefault (boolean): Boolean indicating the default Group. Any device that does not have a group explicitly assigned will upgrade with this group
        - description (string): Description of the Staged Upgrade Group. Length must be 1 to 255 characters
        - assignedDevices (object): The devices and Switch Stacks assigned to the Group
        """

        kwargs.update(locals())

        metadata = {
            'tags': ['networks', 'configure', 'firmwareUpgrades', 'staged', 'groups'],
            'operation': 'createNetworkFirmwareUpgradesStagedGroup'
        }
        resource = f'/networks/{networkId}/firmwareUpgrades/staged/groups'

        body_params = ['name', 'description', 'isDefault', 'assignedDevices', ]
        payload = {k.strip(): v for k, v in kwargs.items() if k.strip() in body_params}
        action = {
            "resource": resource,
            "operation": "create",
            "body": payload
        }
        return action
        





    def deleteNetworkFirmwareUpgradesStagedGroup(self, networkId: str, groupId: str):
        """
        **Delete a Staged Upgrade Group**
        https://developer.cisco.com/meraki/api-v1/#!delete-network-firmware-upgrades-staged-group

        - networkId (string): Network ID
        - groupId (string): Group ID
        """

        metadata = {
            'tags': ['networks', 'configure', 'firmwareUpgrades', 'staged', 'groups'],
            'operation': 'deleteNetworkFirmwareUpgradesStagedGroup'
        }
        resource = f'/networks/{networkId}/firmwareUpgrades/staged/groups/{groupId}'

        action = {
            "resource": resource,
            "operation": "destroy",
        }
        return action
        





    def batchNetworkFloorPlansAutoLocateJobs(self, networkId: str, jobs: list):
        """
        **Schedule auto locate jobs for one or more floor plans in a network**
        https://developer.cisco.com/meraki/api-v1/#!batch-network-floor-plans-auto-locate-jobs

        - networkId (string): Network ID
        - jobs (array): The list of auto locate jobs to be scheduled. Up to 100 jobs can be provided in a request.
        """

        kwargs = locals()

        metadata = {
            'tags': ['networks', 'configure', 'floorPlans', 'autoLocate', 'jobs'],
            'operation': 'batchNetworkFloorPlansAutoLocateJobs'
        }
        resource = f'/networks/{networkId}/floorPlans/autoLocate/jobs/batch'

        body_params = ['jobs', ]
        payload = {k.strip(): v for k, v in kwargs.items() if k.strip() in body_params}
        action = {
            "resource": resource,
            "operation": "batch_create",
            "body": payload
        }
        return action
        





    def cancelNetworkFloorPlansAutoLocateJob(self, networkId: str, jobId: str):
        """
        **Cancel a scheduled or running auto locate job**
        https://developer.cisco.com/meraki/api-v1/#!cancel-network-floor-plans-auto-locate-job

        - networkId (string): Network ID
        - jobId (string): Job ID
        """

        metadata = {
            'tags': ['networks', 'configure', 'floorPlans', 'autoLocate', 'jobs'],
            'operation': 'cancelNetworkFloorPlansAutoLocateJob'
        }
        resource = f'/networks/{networkId}/floorPlans/autoLocate/jobs/{jobId}/cancel'

        action = {
            "resource": resource,
            "operation": "cancel",
        }
        return action
        





    def publishNetworkFloorPlansAutoLocateJob(self, networkId: str, jobId: str, **kwargs):
        """
        **Update the status of a finished auto locate job to be published, and update device locations**
        https://developer.cisco.com/meraki/api-v1/#!publish-network-floor-plans-auto-locate-job

        - networkId (string): Network ID
        - jobId (string): Job ID
        - devices (array): The list of devices to publish positions for
        """

        kwargs.update(locals())

        metadata = {
            'tags': ['networks', 'configure', 'floorPlans', 'autoLocate', 'jobs'],
            'operation': 'publishNetworkFloorPlansAutoLocateJob'
        }
        resource = f'/networks/{networkId}/floorPlans/autoLocate/jobs/{jobId}/publish'

        body_params = ['devices', ]
        payload = {k.strip(): v for k, v in kwargs.items() if k.strip() in body_params}
        action = {
            "resource": resource,
            "operation": "publish",
            "body": payload
        }
        return action
        





    def recalculateNetworkFloorPlansAutoLocateJob(self, networkId: str, jobId: str, **kwargs):
        """
        **Trigger auto locate recalculation for a job, and optionally set anchors**
        https://developer.cisco.com/meraki/api-v1/#!recalculate-network-floor-plans-auto-locate-job

        - networkId (string): Network ID
        - jobId (string): Job ID
        - devices (array): The list of devices to update anchor positions for
        """

        kwargs.update(locals())

        metadata = {
            'tags': ['networks', 'configure', 'floorPlans', 'autoLocate', 'jobs'],
            'operation': 'recalculateNetworkFloorPlansAutoLocateJob'
        }
        resource = f'/networks/{networkId}/floorPlans/autoLocate/jobs/{jobId}/recalculate'

        body_params = ['devices', ]
        payload = {k.strip(): v for k, v in kwargs.items() if k.strip() in body_params}
        action = {
            "resource": resource,
            "operation": "recalculate",
            "body": payload
        }
        return action
        





    def batchNetworkFloorPlansDevicesUpdate(self, networkId: str, assignments: list):
        """
        **Update floorplan assignments for a batch of devices**
        https://developer.cisco.com/meraki/api-v1/#!batch-network-floor-plans-devices-update

        - networkId (string): Network ID
        - assignments (array): List of floorplan assignments to update. Up to 100 floor plan assignments can be provided in a request.
        """

        kwargs = locals()

        metadata = {
            'tags': ['networks', 'configure', 'floorPlans', 'devices'],
            'operation': 'batchNetworkFloorPlansDevicesUpdate'
        }
        resource = f'/networks/{networkId}/floorPlans/devices/batchUpdate'

        body_params = ['assignments', ]
        payload = {k.strip(): v for k, v in kwargs.items() if k.strip() in body_params}
        action = {
            "resource": resource,
            "operation": "batch_update",
            "body": payload
        }
        return action
        





    def updateNetworkFloorPlan(self, networkId: str, floorPlanId: str, **kwargs):
        """
        **Update a floor plan's geolocation and other meta data**
        https://developer.cisco.com/meraki/api-v1/#!update-network-floor-plan

        - networkId (string): Network ID
        - floorPlanId (string): Floor plan ID
        - name (string): The name of your floor plan.
        - center (object): The longitude and latitude of the center of your floor plan. If you want to change the geolocation data of your floor plan, either the 'center' or two adjacent corners (e.g. 'topLeftCorner' and 'bottomLeftCorner') must be specified. If 'center' is specified, the floor plan is placed over that point with no rotation. If two adjacent corners are specified, the floor plan is rotated to line up with the two specified points. The aspect ratio of the floor plan's image is preserved regardless of which corners/center are specified. (This means if that more than two corners are specified, only two corners may be used to preserve the floor plan's aspect ratio.). No two points can have the same latitude, longitude pair.
        - bottomLeftCorner (object): The longitude and latitude of the bottom left corner of your floor plan.
        - bottomRightCorner (object): The longitude and latitude of the bottom right corner of your floor plan.
        - topLeftCorner (object): The longitude and latitude of the top left corner of your floor plan.
        - topRightCorner (object): The longitude and latitude of the top right corner of your floor plan.
        - floorNumber (number): The floor number of the floors within the building
        - imageContents (string): The file contents (a base 64 encoded string) of your new image. Supported formats are PNG, GIF, and JPG. Note that all images are saved as PNG files, regardless of the format they are uploaded in. If you upload a new image, and you do NOT specify any new geolocation fields ('center, 'topLeftCorner', etc), the floor plan will be recentered with no rotation in order to maintain the aspect ratio of your new image.
        """

        kwargs.update(locals())

        metadata = {
            'tags': ['networks', 'configure', 'floorPlans'],
            'operation': 'updateNetworkFloorPlan'
        }
        resource = f'/networks/{networkId}/floorPlans/{floorPlanId}'

        body_params = ['name', 'center', 'bottomLeftCorner', 'bottomRightCorner', 'topLeftCorner', 'topRightCorner', 'floorNumber', 'imageContents', ]
        payload = {k.strip(): v for k, v in kwargs.items() if k.strip() in body_params}
        action = {
            "resource": resource,
            "operation": "update",
            "body": payload
        }
        return action
        





    def deleteNetworkFloorPlan(self, networkId: str, floorPlanId: str):
        """
        **Destroy a floor plan**
        https://developer.cisco.com/meraki/api-v1/#!delete-network-floor-plan

        - networkId (string): Network ID
        - floorPlanId (string): Floor plan ID
        """

        metadata = {
            'tags': ['networks', 'configure', 'floorPlans'],
            'operation': 'deleteNetworkFloorPlan'
        }
        resource = f'/networks/{networkId}/floorPlans/{floorPlanId}'

        action = {
            "resource": resource,
            "operation": "destroy",
        }
        return action
        





    def createNetworkGroupPolicy(self, networkId: str, name: str, **kwargs):
        """
        **Create a group policy**
        https://developer.cisco.com/meraki/api-v1/#!create-network-group-policy

        - networkId (string): Network ID
        - name (string): The name for your group policy. Required.
        - scheduling (object):     The schedule for the group policy. Schedules are applied to days of the week.

        - bandwidth (object):     The bandwidth settings for clients bound to your group policy.

        - firewallAndTrafficShaping (object):     The firewall and traffic shaping rules and settings for your policy.

        - contentFiltering (object): The content filtering settings for your group policy
        - splashAuthSettings (string): Whether clients bound to your policy will bypass splash authorization or behave according to the network's rules. Can be one of 'network default' or 'bypass'. Only available if your network has a wireless configuration.
        - vlanTagging (object): The VLAN tagging settings for your group policy. Only available if your network has a wireless configuration.
        - bonjourForwarding (object): The Bonjour settings for your group policy. Only valid if your network has a wireless configuration.
        """

        kwargs.update(locals())

        if 'splashAuthSettings' in kwargs:
            options = ['bypass', 'network default']
            assert kwargs['splashAuthSettings'] in options, f'''"splashAuthSettings" cannot be "{kwargs['splashAuthSettings']}", & must be set to one of: {options}'''

        metadata = {
            'tags': ['networks', 'configure', 'groupPolicies'],
            'operation': 'createNetworkGroupPolicy'
        }
        resource = f'/networks/{networkId}/groupPolicies'

        body_params = ['name', 'scheduling', 'bandwidth', 'firewallAndTrafficShaping', 'contentFiltering', 'splashAuthSettings', 'vlanTagging', 'bonjourForwarding', ]
        payload = {k.strip(): v for k, v in kwargs.items() if k.strip() in body_params}
        action = {
            "resource": resource,
            "operation": "create",
            "body": payload
        }
        return action
        





    def updateNetworkGroupPolicy(self, networkId: str, groupPolicyId: str, **kwargs):
        """
        **Update a group policy**
        https://developer.cisco.com/meraki/api-v1/#!update-network-group-policy

        - networkId (string): Network ID
        - groupPolicyId (string): Group policy ID
        - name (string): The name for your group policy.
        - scheduling (object):     The schedule for the group policy. Schedules are applied to days of the week.

        - bandwidth (object):     The bandwidth settings for clients bound to your group policy.

        - firewallAndTrafficShaping (object):     The firewall and traffic shaping rules and settings for your policy.

        - contentFiltering (object): The content filtering settings for your group policy
        - splashAuthSettings (string): Whether clients bound to your policy will bypass splash authorization or behave according to the network's rules. Can be one of 'network default' or 'bypass'. Only available if your network has a wireless configuration.
        - vlanTagging (object): The VLAN tagging settings for your group policy. Only available if your network has a wireless configuration.
        - bonjourForwarding (object): The Bonjour settings for your group policy. Only valid if your network has a wireless configuration.
        """

        kwargs.update(locals())

        if 'splashAuthSettings' in kwargs:
            options = ['bypass', 'network default']
            assert kwargs['splashAuthSettings'] in options, f'''"splashAuthSettings" cannot be "{kwargs['splashAuthSettings']}", & must be set to one of: {options}'''

        metadata = {
            'tags': ['networks', 'configure', 'groupPolicies'],
            'operation': 'updateNetworkGroupPolicy'
        }
        resource = f'/networks/{networkId}/groupPolicies/{groupPolicyId}'

        body_params = ['name', 'scheduling', 'bandwidth', 'firewallAndTrafficShaping', 'contentFiltering', 'splashAuthSettings', 'vlanTagging', 'bonjourForwarding', ]
        payload = {k.strip(): v for k, v in kwargs.items() if k.strip() in body_params}
        action = {
            "resource": resource,
            "operation": "update",
            "body": payload
        }
        return action
        





    def deleteNetworkGroupPolicy(self, networkId: str, groupPolicyId: str, **kwargs):
        """
        **Delete a group policy**
        https://developer.cisco.com/meraki/api-v1/#!delete-network-group-policy

        - networkId (string): Network ID
        - groupPolicyId (string): Group policy ID
        - force (boolean): If true, the system deletes the GP even if there are active clients using the GP. After deletion, active clients that were assigned to that Group Policy will be left without any policy applied. Default is false.
        """

        kwargs.update(locals())

        metadata = {
            'tags': ['networks', 'configure', 'groupPolicies'],
            'operation': 'deleteNetworkGroupPolicy'
        }
        resource = f'/networks/{networkId}/groupPolicies/{groupPolicyId}'

        action = {
            "resource": resource,
            "operation": "destroy",
        }
        return action
        





    def createNetworkMerakiAuthUser(self, networkId: str, email: str, authorizations: list, **kwargs):
        """
        **Authorize a user configured with Meraki Authentication for a network (currently supports 802.1X, splash guest, and client VPN users, and currently, organizations have a 50,000 user cap)**
        https://developer.cisco.com/meraki/api-v1/#!create-network-meraki-auth-user

        - networkId (string): Network ID
        - email (string): Email address of the user
        - authorizations (array): Authorization zones and expiration dates for the user.
        - name (string): Name of the user. Only required If the user is not a Dashboard administrator.
        - password (string): The password for this user account. Only required If the user is not a Dashboard administrator.
        - accountType (string): Authorization type for user. Can be 'Guest' or '802.1X' for wireless networks, or 'Client VPN' for MX networks. Defaults to '802.1X'.
        - emailPasswordToUser (boolean): Whether or not Meraki should email the password to user. Default is false.
        - isAdmin (boolean): Whether or not the user is a Dashboard administrator.
        """

        kwargs.update(locals())

        if 'accountType' in kwargs:
            options = ['802.1X', 'Client VPN', 'Guest']
            assert kwargs['accountType'] in options, f'''"accountType" cannot be "{kwargs['accountType']}", & must be set to one of: {options}'''

        metadata = {
            'tags': ['networks', 'configure', 'merakiAuthUsers'],
            'operation': 'createNetworkMerakiAuthUser'
        }
        resource = f'/networks/{networkId}/merakiAuthUsers'

        body_params = ['email', 'name', 'password', 'accountType', 'emailPasswordToUser', 'isAdmin', 'authorizations', ]
        payload = {k.strip(): v for k, v in kwargs.items() if k.strip() in body_params}
        action = {
            "resource": resource,
            "operation": "create",
            "body": payload
        }
        return action
        





    def deleteNetworkMerakiAuthUser(self, networkId: str, merakiAuthUserId: str, **kwargs):
        """
        **Delete an 802.1X RADIUS user, or deauthorize and optionally delete a splash guest or client VPN user.**
        https://developer.cisco.com/meraki/api-v1/#!delete-network-meraki-auth-user

        - networkId (string): Network ID
        - merakiAuthUserId (string): Meraki auth user ID
        - delete (boolean): If the ID supplied is for a splash guest or client VPN user, and that user is not authorized for any other networks in the organization, then also delete the user. 802.1X RADIUS users are always deleted regardless of this optional attribute.
        """

        kwargs.update(locals())

        metadata = {
            'tags': ['networks', 'configure', 'merakiAuthUsers'],
            'operation': 'deleteNetworkMerakiAuthUser'
        }
        resource = f'/networks/{networkId}/merakiAuthUsers/{merakiAuthUserId}'

        action = {
            "resource": resource,
            "operation": "destroy",
        }
        return action
        





    def updateNetworkMerakiAuthUser(self, networkId: str, merakiAuthUserId: str, **kwargs):
        """
        **Update a user configured with Meraki Authentication (currently, 802.1X RADIUS, splash guest, and client VPN users can be updated)**
        https://developer.cisco.com/meraki/api-v1/#!update-network-meraki-auth-user

        - networkId (string): Network ID
        - merakiAuthUserId (string): Meraki auth user ID
        - name (string): Name of the user. Only allowed If the user is not Dashboard administrator.
        - password (string): The password for this user account. Only allowed If the user is not Dashboard administrator.
        - emailPasswordToUser (boolean): Whether or not Meraki should email the password to user. Default is false.
        - authorizations (array): Authorization zones and expiration dates for the user.
        """

        kwargs.update(locals())

        metadata = {
            'tags': ['networks', 'configure', 'merakiAuthUsers'],
            'operation': 'updateNetworkMerakiAuthUser'
        }
        resource = f'/networks/{networkId}/merakiAuthUsers/{merakiAuthUserId}'

        body_params = ['name', 'password', 'emailPasswordToUser', 'authorizations', ]
        payload = {k.strip(): v for k, v in kwargs.items() if k.strip() in body_params}
        action = {
            "resource": resource,
            "operation": "update",
            "body": payload
        }
        return action
        





    def createNetworkMqttBroker(self, networkId: str, name: str, host: str, port: int, **kwargs):
        """
        **Add an MQTT broker**
        https://developer.cisco.com/meraki/api-v1/#!create-network-mqtt-broker

        - networkId (string): Network ID
        - name (string): Name of the MQTT broker.
        - host (string): Host name/IP address where the MQTT broker runs.
        - port (integer): Host port though which the MQTT broker can be reached.
        - security (object): Security settings of the MQTT broker.
        - authentication (object): Authentication settings of the MQTT broker
        """

        kwargs.update(locals())

        metadata = {
            'tags': ['networks', 'configure', 'mqttBrokers'],
            'operation': 'createNetworkMqttBroker'
        }
        resource = f'/networks/{networkId}/mqttBrokers'

        body_params = ['name', 'host', 'port', 'security', 'authentication', ]
        payload = {k.strip(): v for k, v in kwargs.items() if k.strip() in body_params}
        action = {
            "resource": resource,
            "operation": "create",
            "body": payload
        }
        return action
        





    def updateNetworkMqttBroker(self, networkId: str, mqttBrokerId: str, **kwargs):
        """
        **Update an MQTT broker**
        https://developer.cisco.com/meraki/api-v1/#!update-network-mqtt-broker

        - networkId (string): Network ID
        - mqttBrokerId (string): Mqtt broker ID
        - name (string): Name of the MQTT broker.
        - host (string): Host name/IP address where the MQTT broker runs.
        - port (integer): Host port though which the MQTT broker can be reached.
        - security (object): Security settings of the MQTT broker.
        - authentication (object): Authentication settings of the MQTT broker
        """

        kwargs.update(locals())

        metadata = {
            'tags': ['networks', 'configure', 'mqttBrokers'],
            'operation': 'updateNetworkMqttBroker'
        }
        resource = f'/networks/{networkId}/mqttBrokers/{mqttBrokerId}'

        body_params = ['name', 'host', 'port', 'security', 'authentication', ]
        payload = {k.strip(): v for k, v in kwargs.items() if k.strip() in body_params}
        action = {
            "resource": resource,
            "operation": "update",
            "body": payload
        }
        return action
        





    def deleteNetworkMqttBroker(self, networkId: str, mqttBrokerId: str):
        """
        **Delete an MQTT broker**
        https://developer.cisco.com/meraki/api-v1/#!delete-network-mqtt-broker

        - networkId (string): Network ID
        - mqttBrokerId (string): Mqtt broker ID
        """

        metadata = {
            'tags': ['networks', 'configure', 'mqttBrokers'],
            'operation': 'deleteNetworkMqttBroker'
        }
        resource = f'/networks/{networkId}/mqttBrokers/{mqttBrokerId}'

        action = {
            "resource": resource,
            "operation": "destroy",
        }
        return action
        





    def updateNetworkSettings(self, networkId: str, **kwargs):
        """
        **Update the settings for a network**
        https://developer.cisco.com/meraki/api-v1/#!update-network-settings

        - networkId (string): Network ID
        - localStatusPageEnabled (boolean): Enables / disables the local device status pages (<a target='_blank' href='http://my.meraki.com/'>my.meraki.com, </a><a target='_blank' href='http://ap.meraki.com/'>ap.meraki.com, </a><a target='_blank' href='http://switch.meraki.com/'>switch.meraki.com, </a><a target='_blank' href='http://wired.meraki.com/'>wired.meraki.com</a>). Optional (defaults to false)
        - remoteStatusPageEnabled (boolean): Enables / disables access to the device status page (<a target='_blank'>http://[device's LAN IP])</a>. Optional. Can only be set if localStatusPageEnabled is set to true
        - localStatusPage (object): A hash of Local Status page(s)' authentication options applied to the Network.
        - securePort (object): A hash of SecureConnect options applied to the Network.
        - namedVlans (object): A hash of Named VLANs options applied to the Network.
        """

        kwargs.update(locals())

        metadata = {
            'tags': ['networks', 'configure', 'settings'],
            'operation': 'updateNetworkSettings'
        }
        resource = f'/networks/{networkId}/settings'

        body_params = ['localStatusPageEnabled', 'remoteStatusPageEnabled', 'localStatusPage', 'securePort', 'namedVlans', ]
        payload = {k.strip(): v for k, v in kwargs.items() if k.strip() in body_params}
        action = {
            "resource": resource,
            "operation": "update",
            "body": payload
        }
        return action
        





    def splitNetwork(self, networkId: str):
        """
        **Split a combined network into individual networks for each type of device**
        https://developer.cisco.com/meraki/api-v1/#!split-network

        - networkId (string): Network ID
        """

        metadata = {
            'tags': ['networks', 'configure'],
            'operation': 'splitNetwork'
        }
        resource = f'/networks/{networkId}/split'

        action = {
            "resource": resource,
            "operation": "split",
        }
        return action
        





    def unbindNetwork(self, networkId: str, **kwargs):
        """
        **Unbind a network from a template.**
        https://developer.cisco.com/meraki/api-v1/#!unbind-network

        - networkId (string): Network ID
        - retainConfigs (boolean): Optional boolean to retain all the current configs given by the template.
        """

        kwargs.update(locals())

        metadata = {
            'tags': ['networks', 'configure'],
            'operation': 'unbindNetwork'
        }
        resource = f'/networks/{networkId}/unbind'

        body_params = ['retainConfigs', ]
        payload = {k.strip(): v for k, v in kwargs.items() if k.strip() in body_params}
        action = {
            "resource": resource,
            "operation": "unbind",
            "body": payload
        }
        return action
        





    def createNetworkVlanProfile(self, networkId: str, name: str, vlanNames: list, vlanGroups: list, iname: str):
        """
        **Create a VLAN profile for a network**
        https://developer.cisco.com/meraki/api-v1/#!create-network-vlan-profile

        - networkId (string): Network ID
        - name (string): Name of the profile, string length must be from 1 to 255 characters
        - vlanNames (array): An array of named VLANs
        - vlanGroups (array): An array of VLAN groups
        - iname (string): IName of the profile
        """

        kwargs = locals()

        metadata = {
            'tags': ['networks', 'configure', 'vlanProfiles'],
            'operation': 'createNetworkVlanProfile'
        }
        resource = f'/networks/{networkId}/vlanProfiles'

        body_params = ['name', 'vlanNames', 'vlanGroups', 'iname', ]
        payload = {k.strip(): v for k, v in kwargs.items() if k.strip() in body_params}
        action = {
            "resource": resource,
            "operation": "create",
            "body": payload
        }
        return action
        





    def deleteNetworkVlanProfile(self, networkId: str, iname: str):
        """
        **Delete a VLAN profile of a network**
        https://developer.cisco.com/meraki/api-v1/#!delete-network-vlan-profile

        - networkId (string): Network ID
        - iname (string): Iname
        """

        metadata = {
            'tags': ['networks', 'configure', 'vlanProfiles'],
            'operation': 'deleteNetworkVlanProfile'
        }
        resource = f'/networks/{networkId}/vlanProfiles/{iname}'

        action = {
            "resource": resource,
            "operation": "destroy",
        }
        return action
        





    def createNetworkWebhooksPayloadTemplate(self, networkId: str, name: str, **kwargs):
        """
        **Create a webhook payload template for a network**
        https://developer.cisco.com/meraki/api-v1/#!create-network-webhooks-payload-template

        - networkId (string): Network ID
        - name (string): The name of the new template
        - body (string): The liquid template used for the body of the webhook message. Either `body` or `bodyFile` must be specified.
        - headers (array): The liquid template used with the webhook headers.
        - bodyFile (string): A Base64 encoded file containing liquid template used for the body of the webhook message. Either `body` or `bodyFile` must be specified.
        - headersFile (string): A Base64 encoded file containing the liquid template used with the webhook headers.
        """

        kwargs.update(locals())

        metadata = {
            'tags': ['networks', 'configure', 'webhooks', 'payloadTemplates'],
            'operation': 'createNetworkWebhooksPayloadTemplate'
        }
        resource = f'/networks/{networkId}/webhooks/payloadTemplates'

        body_params = ['name', 'body', 'headers', 'bodyFile', 'headersFile', ]
        payload = {k.strip(): v for k, v in kwargs.items() if k.strip() in body_params}
        action = {
            "resource": resource,
            "operation": "create",
            "body": payload
        }
        return action
        





    def deleteNetworkWebhooksPayloadTemplate(self, networkId: str, payloadTemplateId: str):
        """
        **Destroy a webhook payload template for a network. Does not work for included templates ('wpt_00001', 'wpt_00002', 'wpt_00003', 'wpt_00004', 'wpt_00005' or 'wpt_00006')**
        https://developer.cisco.com/meraki/api-v1/#!delete-network-webhooks-payload-template

        - networkId (string): Network ID
        - payloadTemplateId (string): Payload template ID
        """

        metadata = {
            'tags': ['networks', 'configure', 'webhooks', 'payloadTemplates'],
            'operation': 'deleteNetworkWebhooksPayloadTemplate'
        }
        resource = f'/networks/{networkId}/webhooks/payloadTemplates/{payloadTemplateId}'

        action = {
            "resource": resource,
            "operation": "destroy",
        }
        return action
        





    def updateNetworkWebhooksPayloadTemplate(self, networkId: str, payloadTemplateId: str, **kwargs):
        """
        **Update a webhook payload template for a network**
        https://developer.cisco.com/meraki/api-v1/#!update-network-webhooks-payload-template

        - networkId (string): Network ID
        - payloadTemplateId (string): Payload template ID
        - name (string): The name of the template
        - body (string): The liquid template used for the body of the webhook message.
        - headers (array): The liquid template used with the webhook headers.
        - bodyFile (string): A file containing liquid template used for the body of the webhook message.
        - headersFile (string): A file containing the liquid template used with the webhook headers.
        """

        kwargs.update(locals())

        metadata = {
            'tags': ['networks', 'configure', 'webhooks', 'payloadTemplates'],
            'operation': 'updateNetworkWebhooksPayloadTemplate'
        }
        resource = f'/networks/{networkId}/webhooks/payloadTemplates/{payloadTemplateId}'

        body_params = ['name', 'body', 'headers', 'bodyFile', 'headersFile', ]
        payload = {k.strip(): v for k, v in kwargs.items() if k.strip() in body_params}
        action = {
            "resource": resource,
            "operation": "update",
            "body": payload
        }
        return action
        






# Package Constants

# Meraki dashboard API key, set either at instantiation or as an environment variable
API_KEY_ENVIRONMENT_VARIABLE = "MERAKI_DASHBOARD_API_KEY"

# Base URL preceding all endpoint resources
DEFAULT_BASE_URL = "https://api.meraki.com/api/v1"

# Alternate base URLs
CANADA_BASE_URL = "https://api.meraki.ca/api/v1"
CHINA_BASE_URL = "https://api.meraki.cn/api/v1"
INDIA_BASE_URL = "https://api.meraki.in/api/v1"
UNITED_STATES_FED_BASE_URL = "https://api.gov-meraki.com/api/v1"

# Maximum number of seconds for each API call
SINGLE_REQUEST_TIMEOUT = 60

# Path for TLS/SSL certificate verification if behind local proxy
CERTIFICATE_PATH = ""

# Proxy server and port, if needed, for HTTPS
REQUESTS_PROXY = ""

# Retry if 429 rate limit error encountered?
# Please note, setting to False means your application will not retry upon a 429. Not intended for production apps.
WAIT_ON_RATE_LIMIT = True

# Nginx 429 retry wait time
NGINX_429_RETRY_WAIT_TIME = 60

# Action batch concurrency error retry wait time
ACTION_BATCH_RETRY_WAIT_TIME = 60

# Network deletion concurrency error retry wait time
NETWORK_DELETE_RETRY_WAIT_TIME = 240

# Retry if encountering other 4XX error (besides 429)?
RETRY_4XX_ERROR = False

# Other 4XX error retry wait time
RETRY_4XX_ERROR_WAIT_TIME = 60

# Retry up to this many times when encountering 429s or other server-side errors
MAXIMUM_RETRIES = 2

# Create an output log file?
OUTPUT_LOG = True

# Path to output log; by default, working directory of script if not specified
LOG_PATH = ""

# Log file name appended with date and timestamp
LOG_FILE_PREFIX = "meraki_api_"

# Print output logging to console?
PRINT_TO_CONSOLE = True

# Disable all logging? You're on your own then!
SUPPRESS_LOGGING = False

# You might integrate the library in an application with a predefined logging scheme. If so, you may not need the
# library's default logging handlers, formatters etc.--instead, you can inherit an external logger instance.
INHERIT_LOGGING_CONFIG = False

# Use iterator for pages. May offer improved performance in some instances. Off by default for backwards compatibility.
USE_ITERATOR_FOR_GET_PAGES = False

# Simulate POST/PUT/DELETE calls to prevent changes?
SIMULATE_API_CALLS = False

# Number of concurrent API requests for asynchronous class
AIO_MAXIMUM_CONCURRENT_REQUESTS = 8

# Legacy partner identifier for API usage tracking; can also be set as an environment variable BE_GEO_ID
# This is no longer used. Please use MERAKI_PYTHON_SDK_CALLER instead.
BE_GEO_ID = ""

# Optional identifier for API usage tracking; can also be set as an environment variable MERAKI_PYTHON_SDK_CALLER
# It's good practice to use this to identify your application using the format:
# CamelCasedApplicationName/OptionalVersionNumber CamelCasedVendorName
# Please note:
# 1. Application name precedes vendor name in all cases.
# 2. If your application or vendor name normally contains spaces or special casing, you should omit them in favor of
#    normal CamelCasing here.
# 3. The slash and version number are optional. Leave both out if you like.
# 4. The slash is a forward slash, '/' -- not a backslash.
# 5. Don't use the 'Meraki' or 'Cisco' names in your application name here. Maybe in general? I'm a config file, not a
#    lawyer.
# Example 1: if your application is named 'Mambo', version number is 5.0, and your vendor/company name is Vega, then
# you would use, at minimum: 'Mambo Vega'. Optionally: 'Mambo/5.0 Vega'.
# Example 2: if your application is named 'Sunshine Rainbows', and company name is 'hunter2 for Life', and if you
# don't want to report version number, then you would use, at minimum: 'SunshineRainbows hunter2ForLife'
# The choice is yours as long as you follow the format. You should **not** include other information in this string.
# If you are an official ecosystem partner, this is required.
# For more guidance, please refer to https://developer.cisco.com/meraki/api-v1/user-agents-overview/
MERAKI_PYTHON_SDK_CALLER = ""



import logging
import os

from meraki.aio.api.administered import AsyncAdministered
from meraki.aio.api.appliance import AsyncAppliance
from meraki.aio.api.camera import AsyncCamera
from meraki.aio.api.cellularGateway import AsyncCellularGateway
from meraki.aio.api.devices import AsyncDevices
from meraki.aio.api.insight import AsyncInsight
from meraki.aio.api.licensing import AsyncLicensing
from meraki.aio.api.networks import AsyncNetworks
from meraki.aio.api.organizations import AsyncOrganizations
from meraki.aio.api.sensor import AsyncSensor
from meraki.aio.api.sm import AsyncSm
from meraki.aio.api.switch import AsyncSwitch
from meraki.aio.api.wireless import AsyncWireless
from meraki.aio.rest_session import *
# Batch class imports
from meraki.api.batch import Batch
# Config import
from meraki.config import (
    API_KEY_ENVIRONMENT_VARIABLE,
    DEFAULT_BASE_URL,
    SINGLE_REQUEST_TIMEOUT,
    CERTIFICATE_PATH,
    REQUESTS_PROXY,
    WAIT_ON_RATE_LIMIT,
    NGINX_429_RETRY_WAIT_TIME,
    ACTION_BATCH_RETRY_WAIT_TIME,
    NETWORK_DELETE_RETRY_WAIT_TIME,
    RETRY_4XX_ERROR,
    RETRY_4XX_ERROR_WAIT_TIME,
    MAXIMUM_RETRIES,
    OUTPUT_LOG,
    LOG_PATH,
    LOG_FILE_PREFIX,
    PRINT_TO_CONSOLE,
    SUPPRESS_LOGGING,
    INHERIT_LOGGING_CONFIG,
    SIMULATE_API_CALLS,
    BE_GEO_ID,
    MERAKI_PYTHON_SDK_CALLER,
    USE_ITERATOR_FOR_GET_PAGES,
    AIO_MAXIMUM_CONCURRENT_REQUESTS,
)


class AsyncDashboardAPI:
    """
    **Creates a persistent Meraki dashboard API session**

    - api_key (string): API key generated in dashboard; can also be set as an environment variable MERAKI_DASHBOARD_API_KEY
    - base_url (string): preceding all endpoint resources
    - single_request_timeout (integer): maximum number of seconds for each API call
    - certificate_path (string): path for TLS/SSL certificate verification if behind local proxy
    - requests_proxy (string): proxy server and port, if needed, for HTTPS
    - wait_on_rate_limit (boolean): retry if 429 rate limit error encountered?
    - nginx_429_retry_wait_time (integer): Nginx 429 retry wait time
    - action_batch_retry_wait_time (integer): action batch concurrency error retry wait time
    - network_delete_retry_wait_time (integer): network deletion concurrency error retry wait time
    - retry_4xx_error (boolean): retry if encountering other 4XX error (besides 429)?
    - retry_4xx_error_wait_time (integer): other 4XX error retry wait time
    - maximum_retries (integer): retry up to this many times when encountering 429s or other server-side errors
    - output_log (boolean): create an output log file?
    - log_path (string): path to output log; by default, working directory of script if not specified
    - log_file_prefix (string): log file name appended with date and timestamp
    - print_console (boolean): print logging output to console?
    - suppress_logging (boolean): disable all logging? you're on your own then!
    - inherit_logging_config (boolean): Inherits your own logger instance
    - simulate (boolean): simulate POST/PUT/DELETE calls to prevent changes?
    - maximum_concurrent_requests (integer): number of concurrent API requests for asynchronous class
    - be_geo_id (string): optional partner identifier for API usage tracking; can also be set as an environment variable BE_GEO_ID
    - caller (string): optional identifier for API usage tracking; can also be set as an environment variable MERAKI_PYTHON_SDK_CALLER
    - use_iterator_for_get_pages (boolean): list* methods will return an iterator with each object instead of a complete list with all items
    """

    def __init__(self,
                 api_key=None,
                 base_url=DEFAULT_BASE_URL,
                 single_request_timeout=SINGLE_REQUEST_TIMEOUT,
                 certificate_path=CERTIFICATE_PATH,
                 requests_proxy=REQUESTS_PROXY,
                 wait_on_rate_limit=WAIT_ON_RATE_LIMIT,
                 nginx_429_retry_wait_time=NGINX_429_RETRY_WAIT_TIME,
                 action_batch_retry_wait_time=ACTION_BATCH_RETRY_WAIT_TIME,
                 network_delete_retry_wait_time=NETWORK_DELETE_RETRY_WAIT_TIME,
                 retry_4xx_error=RETRY_4XX_ERROR,
                 retry_4xx_error_wait_time=RETRY_4XX_ERROR_WAIT_TIME,
                 maximum_retries=MAXIMUM_RETRIES,
                 output_log=OUTPUT_LOG,
                 log_path=LOG_PATH,
                 log_file_prefix=LOG_FILE_PREFIX,
                 print_console=PRINT_TO_CONSOLE,
                 suppress_logging=SUPPRESS_LOGGING,
                 simulate=SIMULATE_API_CALLS,
                 be_geo_id=BE_GEO_ID,
                 caller=MERAKI_PYTHON_SDK_CALLER,
                 use_iterator_for_get_pages=USE_ITERATOR_FOR_GET_PAGES,
                 inherit_logging_config=INHERIT_LOGGING_CONFIG,
                 maximum_concurrent_requests=AIO_MAXIMUM_CONCURRENT_REQUESTS,
                 ):

        # Check API key
        api_key = api_key or os.environ.get(API_KEY_ENVIRONMENT_VARIABLE)
        if not api_key:
            raise APIKeyError()

        # Pull the BE GEO ID from an environment variable if present
        be_geo_id = be_geo_id or os.environ.get('BE_GEO_ID')

        # Pull the caller from an environment variable if present
        caller = caller or os.environ.get('MERAKI_PYTHON_SDK_CALLER')

        use_iterator_for_get_pages = use_iterator_for_get_pages
        inherit_logging_config = inherit_logging_config

        # Configure logging
        if not suppress_logging:
            self._logger = logging.getLogger(__name__)

            if not inherit_logging_config:
                self._logger.setLevel(logging.DEBUG)

                formatter = logging.Formatter(
                    fmt="%(asctime)s %(name)12s: %(levelname)8s > %(message)s",
                    datefmt="%Y-%m-%d %H:%M:%S",
                )
                handler_console = logging.StreamHandler()
                handler_console.setFormatter(formatter)

                if output_log:
                    if log_path and log_path[-1] != "/":
                        log_path += "/"
                    self._log_file = f"{log_path}{log_file_prefix}_log__{datetime.now():%Y-%m-%d_%H-%M-%S}.log"
                    handler_log = logging.FileHandler(filename=self._log_file)
                    handler_log.setFormatter(formatter)

                if output_log and not self._logger.hasHandlers():
                    self._logger.addHandler(handler_log)
                    if print_console:
                        handler_console.setLevel(logging.INFO)
                        self._logger.addHandler(handler_console)
                elif print_console and not self._logger.hasHandlers():
                    self._logger.addHandler(handler_console)
        else:
            self._logger = None

        # Creates the API session
        self._session = AsyncRestSession(
            logger=self._logger,
            api_key=api_key,
            base_url=base_url,
            single_request_timeout=single_request_timeout,
            certificate_path=certificate_path,
            requests_proxy=requests_proxy,
            wait_on_rate_limit=wait_on_rate_limit,
            nginx_429_retry_wait_time=nginx_429_retry_wait_time,
            action_batch_retry_wait_time=action_batch_retry_wait_time,
            network_delete_retry_wait_time=network_delete_retry_wait_time,
            retry_4xx_error=retry_4xx_error,
            retry_4xx_error_wait_time=retry_4xx_error_wait_time,
            maximum_retries=maximum_retries,
            simulate=simulate,
            be_geo_id=be_geo_id,
            caller=caller,
            use_iterator_for_get_pages=use_iterator_for_get_pages,
            maximum_concurrent_requests=maximum_concurrent_requests,
        )

        # API endpoints by section
        self.administered = AsyncAdministered(self._session)
        self.organizations = AsyncOrganizations(self._session)
        self.networks = AsyncNetworks(self._session)
        self.devices = AsyncDevices(self._session)
        self.appliance = AsyncAppliance(self._session)
        self.camera = AsyncCamera(self._session)
        self.cellularGateway = AsyncCellularGateway(self._session)
        self.insight = AsyncInsight(self._session)
        self.licensing = AsyncLicensing(self._session)
        self.sensor = AsyncSensor(self._session)
        self.switch = AsyncSwitch(self._session)
        self.sm = AsyncSm(self._session)
        self.wireless = AsyncWireless(self._session)

        # Batch definitions
        self.batch = Batch()

    async def __aenter__(self):
        return self

    async def __aexit__(self, exc_type, exc, tb):
        await self._session.close()



import urllib


class AsyncLicensing:
    def __init__(self, session):
        super().__init__()
        self._session = session
        


    def getAdministeredLicensingSubscriptionEntitlements(self, **kwargs):
        """
        **Retrieve the list of purchasable entitlements**
        https://developer.cisco.com/meraki/api-v1/#!get-administered-licensing-subscription-entitlements

        - skus (array): Filter to entitlements with the specified SKUs
        """

        kwargs.update(locals())

        metadata = {
            'tags': ['licensing', 'configure', 'subscription', 'entitlements'],
            'operation': 'getAdministeredLicensingSubscriptionEntitlements'
        }
        resource = f'/administered/licensing/subscription/entitlements'

        query_params = ['skus', ]
        params = {k.strip(): v for k, v in kwargs.items() if k.strip() in query_params}

        array_params = ['skus', ]
        for k, v in kwargs.items():
            if k.strip() in array_params:
                params[f'{k.strip()}[]'] = kwargs[f'{k}']
                params.pop(k.strip())

        return self._session.get(metadata, resource, params)
        


    def getAdministeredLicensingSubscriptionSubscriptions(self, organizationIds: list, total_pages=1, direction='next', **kwargs):
        """
        **List available subscriptions**
        https://developer.cisco.com/meraki/api-v1/#!get-administered-licensing-subscription-subscriptions

        - organizationIds (array): Organizations to get associated subscriptions for
        - total_pages (integer or string): use with perPage to get total results up to total_pages*perPage; -1 or "all" for all pages
        - direction (string): direction to paginate, either "next" (default) or "prev" page
        - perPage (integer): The number of entries per page returned. Acceptable range is 3 - 1000. Default is 1000.
        - startingAfter (string): A token used by the server to indicate the start of the page. Often this is a timestamp or an ID but it is not limited to those. This parameter should not be defined by client applications. The link for the first, last, prev, or next page in the HTTP Link header should define it.
        - endingBefore (string): A token used by the server to indicate the end of the page. Often this is a timestamp or an ID but it is not limited to those. This parameter should not be defined by client applications. The link for the first, last, prev, or next page in the HTTP Link header should define it.
        - subscriptionIds (array): List of subscription ids to fetch
        - startDate (string): Filter subscriptions by start date, ISO 8601 format. To filter with a range of dates, use 'startDate[<option>]=?' in the request. Accepted options include lt, gt, lte, gte.
        - endDate (string): Filter subscriptions by end date, ISO 8601 format. To filter with a range of dates, use 'endDate[<option>]=?' in the request. Accepted options include lt, gt, lte, gte.
        - statuses (array): List of statuses that returned subscriptions can have
        - productTypes (array): List of product types that returned subscriptions need to have entitlements for.
        - skus (array): List of SKUs that returned subscriptions need to have entitlements for.
        - name (string): Search for subscription name
        """

        kwargs.update(locals())

        metadata = {
            'tags': ['licensing', 'configure', 'subscription', 'subscriptions'],
            'operation': 'getAdministeredLicensingSubscriptionSubscriptions'
        }
        resource = f'/administered/licensing/subscription/subscriptions'

        query_params = ['perPage', 'startingAfter', 'endingBefore', 'subscriptionIds', 'organizationIds', 'startDate', 'endDate', 'statuses', 'productTypes', 'skus', 'name', ]
        params = {k.strip(): v for k, v in kwargs.items() if k.strip() in query_params}

        array_params = ['subscriptionIds', 'organizationIds', 'statuses', 'productTypes', 'skus', ]
        for k, v in kwargs.items():
            if k.strip() in array_params:
                params[f'{k.strip()}[]'] = kwargs[f'{k}']
                params.pop(k.strip())

        return self._session.get_pages(metadata, resource, params, total_pages, direction)
        


    def claimAdministeredLicensingSubscriptionSubscriptions(self, claimKey: str, organizationId: str, **kwargs):
        """
        **Claim a subscription into an organization.**
        https://developer.cisco.com/meraki/api-v1/#!claim-administered-licensing-subscription-subscriptions

        - claimKey (string): The subscription's claim key
        - organizationId (string): The id of the organization claiming the subscription
        - validate (boolean): Check if the provided claim key is valid and can be claimed into the organization.
        - name (string): Friendly name to identify the subscription
        - description (string): Extra details or notes about the subscription
        """

        kwargs.update(locals())

        metadata = {
            'tags': ['licensing', 'configure', 'subscription', 'subscriptions'],
            'operation': 'claimAdministeredLicensingSubscriptionSubscriptions'
        }
        resource = f'/administered/licensing/subscription/subscriptions/claim'

        body_params = ['claimKey', 'organizationId', 'name', 'description', ]
        payload = {k.strip(): v for k, v in kwargs.items() if k.strip() in body_params}

        return self._session.post(metadata, resource, payload)
        


    def validateAdministeredLicensingSubscriptionSubscriptionsClaimKey(self, claimKey: str):
        """
        **Find a subscription by claim key**
        https://developer.cisco.com/meraki/api-v1/#!validate-administered-licensing-subscription-subscriptions-claim-key

        - claimKey (string): The subscription's claim key
        """

        kwargs = locals()

        metadata = {
            'tags': ['licensing', 'configure', 'subscription', 'subscriptions', 'claimKey'],
            'operation': 'validateAdministeredLicensingSubscriptionSubscriptionsClaimKey'
        }
        resource = f'/administered/licensing/subscription/subscriptions/claimKey/validate'

        body_params = ['claimKey', ]
        payload = {k.strip(): v for k, v in kwargs.items() if k.strip() in body_params}

        return self._session.post(metadata, resource, payload)
        


    def getAdministeredLicensingSubscriptionSubscriptionsComplianceStatuses(self, organizationIds: list, **kwargs):
        """
        **Get compliance status for requested subscriptions**
        https://developer.cisco.com/meraki/api-v1/#!get-administered-licensing-subscription-subscriptions-compliance-statuses

        - organizationIds (array): Organizations to get subscription compliance information for
        - subscriptionIds (array): Subscription ids
        """

        kwargs.update(locals())

        metadata = {
            'tags': ['licensing', 'configure', 'subscription', 'subscriptions', 'compliance', 'statuses'],
            'operation': 'getAdministeredLicensingSubscriptionSubscriptionsComplianceStatuses'
        }
        resource = f'/administered/licensing/subscription/subscriptions/compliance/statuses'

        query_params = ['organizationIds', 'subscriptionIds', ]
        params = {k.strip(): v for k, v in kwargs.items() if k.strip() in query_params}

        array_params = ['organizationIds', 'subscriptionIds', ]
        for k, v in kwargs.items():
            if k.strip() in array_params:
                params[f'{k.strip()}[]'] = kwargs[f'{k}']
                params.pop(k.strip())

        return self._session.get(metadata, resource, params)
        


    def bindAdministeredLicensingSubscriptionSubscription(self, subscriptionId: str, **kwargs):
        """
        **Bind networks to a subscription**
        https://developer.cisco.com/meraki/api-v1/#!bind-administered-licensing-subscription-subscription

        - subscriptionId (string): Subscription ID
        - validate (boolean): Check if the provided networks can be bound to the subscription. Returns any licensing problems and does not commit the results.
        - networkIds (array): List of network ids to bind to the subscription
        """

        kwargs.update(locals())

        metadata = {
            'tags': ['licensing', 'configure', 'subscription', 'subscriptions'],
            'operation': 'bindAdministeredLicensingSubscriptionSubscription'
        }
        subscriptionId = urllib.parse.quote(str(subscriptionId), safe='')
        resource = f'/administered/licensing/subscription/subscriptions/{subscriptionId}/bind'

        body_params = ['networkIds', ]
        payload = {k.strip(): v for k, v in kwargs.items() if k.strip() in body_params}

        return self._session.post(metadata, resource, payload)
        


    def getOrganizationLicensingCotermLicenses(self, organizationId: str, total_pages=1, direction='next', **kwargs):
        """
        **List the licenses in a coterm organization**
        https://developer.cisco.com/meraki/api-v1/#!get-organization-licensing-coterm-licenses

        - organizationId (string): Organization ID
        - total_pages (integer or string): use with perPage to get total results up to total_pages*perPage; -1 or "all" for all pages
        - direction (string): direction to paginate, either "next" (default) or "prev" page
        - perPage (integer): The number of entries per page returned. Acceptable range is 3 - 1000. Default is 1000.
        - startingAfter (string): A token used by the server to indicate the start of the page. Often this is a timestamp or an ID but it is not limited to those. This parameter should not be defined by client applications. The link for the first, last, prev, or next page in the HTTP Link header should define it.
        - endingBefore (string): A token used by the server to indicate the end of the page. Often this is a timestamp or an ID but it is not limited to those. This parameter should not be defined by client applications. The link for the first, last, prev, or next page in the HTTP Link header should define it.
        - invalidated (boolean): Filter for licenses that are invalidated
        - expired (boolean): Filter for licenses that are expired
        """

        kwargs.update(locals())

        metadata = {
            'tags': ['licensing', 'configure', 'coterm', 'licenses'],
            'operation': 'getOrganizationLicensingCotermLicenses'
        }
        organizationId = urllib.parse.quote(str(organizationId), safe='')
        resource = f'/organizations/{organizationId}/licensing/coterm/licenses'

        query_params = ['perPage', 'startingAfter', 'endingBefore', 'invalidated', 'expired', ]
        params = {k.strip(): v for k, v in kwargs.items() if k.strip() in query_params}

        return self._session.get_pages(metadata, resource, params, total_pages, direction)
        


    def moveOrganizationLicensingCotermLicenses(self, organizationId: str, destination: dict, licenses: list):
        """
        **Moves a license to a different organization (coterm only)**
        https://developer.cisco.com/meraki/api-v1/#!move-organization-licensing-coterm-licenses

        - organizationId (string): Organization ID
        - destination (object): Destination data for the license move
        - licenses (array): The list of licenses to move
        """

        kwargs = locals()

        metadata = {
            'tags': ['licensing', 'configure', 'coterm', 'licenses'],
            'operation': 'moveOrganizationLicensingCotermLicenses'
        }
        organizationId = urllib.parse.quote(str(organizationId), safe='')
        resource = f'/organizations/{organizationId}/licensing/coterm/licenses/move'

        body_params = ['destination', 'licenses', ]
        payload = {k.strip(): v for k, v in kwargs.items() if k.strip() in body_params}

        return self._session.post(metadata, resource, payload)
        



import urllib


class ActionBatchCamera(object):
    def __init__(self):
        super(ActionBatchCamera, self).__init__()
        


    def updateDeviceCameraCustomAnalytics(self, serial: str, **kwargs):
        """
        **Update custom analytics settings for a camera**
        https://developer.cisco.com/meraki/api-v1/#!update-device-camera-custom-analytics

        - serial (string): Serial
        - enabled (boolean): Enable custom analytics
        - artifactId (string): The ID of the custom analytics artifact
        - parameters (array): Parameters for the custom analytics workload
        """

        kwargs.update(locals())

        metadata = {
            'tags': ['camera', 'configure', 'customAnalytics'],
            'operation': 'updateDeviceCameraCustomAnalytics'
        }
        resource = f'/devices/{serial}/camera/customAnalytics'

        body_params = ['enabled', 'artifactId', 'parameters', ]
        payload = {k.strip(): v for k, v in kwargs.items() if k.strip() in body_params}
        action = {
            "resource": resource,
            "operation": "update",
            "body": payload
        }
        return action
        





    def updateDeviceCameraQualityAndRetention(self, serial: str, **kwargs):
        """
        **Update quality and retention settings for the given camera**
        https://developer.cisco.com/meraki/api-v1/#!update-device-camera-quality-and-retention

        - serial (string): Serial
        - profileId (string): The ID of a quality and retention profile to assign to the camera. The profile's settings will override all of the per-camera quality and retention settings. If the value of this parameter is null, any existing profile will be unassigned from the camera.
        - motionBasedRetentionEnabled (boolean): Boolean indicating if motion-based retention is enabled(true) or disabled(false) on the camera.
        - audioRecordingEnabled (boolean): Boolean indicating if audio recording is enabled(true) or disabled(false) on the camera
        - restrictedBandwidthModeEnabled (boolean): Boolean indicating if restricted bandwidth is enabled(true) or disabled(false) on the camera. This setting does not apply to MV2 cameras.
        - quality (string): Quality of the camera. Can be one of 'Standard', 'High', 'Enhanced' or 'Ultra'. Not all qualities are supported by every camera model.
        - resolution (string): Resolution of the camera. Can be one of '1280x720', '1920x1080', '1080x1080', '2112x2112', '2880x2880', '2688x1512' or '3840x2160'.Not all resolutions are supported by every camera model.
        - motionDetectorVersion (integer): The version of the motion detector that will be used by the camera. Only applies to Gen 2 cameras. Defaults to v2.
        """

        kwargs.update(locals())

        if 'quality' in kwargs:
            options = ['Enhanced', 'High', 'Standard', 'Ultra']
            assert kwargs['quality'] in options, f'''"quality" cannot be "{kwargs['quality']}", & must be set to one of: {options}'''
        if 'resolution' in kwargs:
            options = ['1080x1080', '1280x720', '1920x1080', '2112x2112', '2688x1512', '2880x2880', '3840x2160']
            assert kwargs['resolution'] in options, f'''"resolution" cannot be "{kwargs['resolution']}", & must be set to one of: {options}'''
        if 'motionDetectorVersion' in kwargs:
            options = [1, 2]
            assert kwargs['motionDetectorVersion'] in options, f'''"motionDetectorVersion" cannot be "{kwargs['motionDetectorVersion']}", & must be set to one of: {options}'''

        metadata = {
            'tags': ['camera', 'configure', 'qualityAndRetention'],
            'operation': 'updateDeviceCameraQualityAndRetention'
        }
        resource = f'/devices/{serial}/camera/qualityAndRetention'

        body_params = ['profileId', 'motionBasedRetentionEnabled', 'audioRecordingEnabled', 'restrictedBandwidthModeEnabled', 'quality', 'resolution', 'motionDetectorVersion', ]
        payload = {k.strip(): v for k, v in kwargs.items() if k.strip() in body_params}
        action = {
            "resource": resource,
            "operation": "update",
            "body": payload
        }
        return action
        





    def updateDeviceCameraSense(self, serial: str, **kwargs):
        """
        **Update sense settings for the given camera**
        https://developer.cisco.com/meraki/api-v1/#!update-device-camera-sense

        - serial (string): Serial
        - senseEnabled (boolean): Boolean indicating if sense(license) is enabled(true) or disabled(false) on the camera
        - mqttBrokerId (string): The ID of the MQTT broker to be enabled on the camera. A value of null will disable MQTT on the camera
        - audioDetection (object): The details of the audio detection config.
        - detectionModelId (string): The ID of the object detection model
        """

        kwargs.update(locals())

        metadata = {
            'tags': ['camera', 'configure', 'sense'],
            'operation': 'updateDeviceCameraSense'
        }
        resource = f'/devices/{serial}/camera/sense'

        body_params = ['senseEnabled', 'mqttBrokerId', 'audioDetection', 'detectionModelId', ]
        payload = {k.strip(): v for k, v in kwargs.items() if k.strip() in body_params}
        action = {
            "resource": resource,
            "operation": "update",
            "body": payload
        }
        return action
        





    def updateDeviceCameraVideoSettings(self, serial: str, **kwargs):
        """
        **Update video settings for the given camera**
        https://developer.cisco.com/meraki/api-v1/#!update-device-camera-video-settings

        - serial (string): Serial
        - externalRtspEnabled (boolean): Boolean indicating if external rtsp stream is exposed
        """

        kwargs.update(locals())

        metadata = {
            'tags': ['camera', 'configure', 'video', 'settings'],
            'operation': 'updateDeviceCameraVideoSettings'
        }
        resource = f'/devices/{serial}/camera/video/settings'

        body_params = ['externalRtspEnabled', ]
        payload = {k.strip(): v for k, v in kwargs.items() if k.strip() in body_params}
        action = {
            "resource": resource,
            "operation": "update",
            "body": payload
        }
        return action
        





    def updateDeviceCameraWirelessProfiles(self, serial: str, ids: dict):
        """
        **Assign wireless profiles to the given camera. Incremental updates are not supported, all profile assignment need to be supplied at once.**
        https://developer.cisco.com/meraki/api-v1/#!update-device-camera-wireless-profiles

        - serial (string): Serial
        - ids (object): The ids of the wireless profile to assign to the given camera
        """

        kwargs = locals()

        metadata = {
            'tags': ['camera', 'configure', 'wirelessProfiles'],
            'operation': 'updateDeviceCameraWirelessProfiles'
        }
        resource = f'/devices/{serial}/camera/wirelessProfiles'

        body_params = ['ids', ]
        payload = {k.strip(): v for k, v in kwargs.items() if k.strip() in body_params}
        action = {
            "resource": resource,
            "operation": "update",
            "body": payload
        }
        return action
        






import urllib


class ActionBatchSm(object):
    def __init__(self):
        super(ActionBatchSm, self).__init__()
        


    def deleteNetworkSmUserAccessDevice(self, networkId: str, userAccessDeviceId: str):
        """
        **Delete a User Access Device**
        https://developer.cisco.com/meraki/api-v1/#!delete-network-sm-user-access-device

        - networkId (string): Network ID
        - userAccessDeviceId (string): User access device ID
        """

        metadata = {
            'tags': ['sm', 'configure', 'userAccessDevices'],
            'operation': 'deleteNetworkSmUserAccessDevice'
        }
        resource = f'/networks/{networkId}/sm/userAccessDevices/{userAccessDeviceId}'

        action = {
            "resource": resource,
            "operation": "destroy",
        }
        return action
        





    def createOrganizationSmAdminsRole(self, organizationId: str, name: str, **kwargs):
        """
        **Create a Limited Access Role**
        https://developer.cisco.com/meraki/api-v1/#!create-organization-sm-admins-role

        - organizationId (string): Organization ID
        - name (string): The name of the Limited Access Role
        - scope (string): The scope of the Limited Access Role
        - tags (array): The tags of the Limited Access Role
        """

        kwargs.update(locals())

        if 'scope' in kwargs:
            options = ['all_tags', 'some', 'without_all_tags', 'without_some']
            assert kwargs['scope'] in options, f'''"scope" cannot be "{kwargs['scope']}", & must be set to one of: {options}'''

        metadata = {
            'tags': ['sm', 'configure', 'admins', 'roles'],
            'operation': 'createOrganizationSmAdminsRole'
        }
        resource = f'/organizations/{organizationId}/sm/admins/roles'

        body_params = ['name', 'scope', 'tags', ]
        payload = {k.strip(): v for k, v in kwargs.items() if k.strip() in body_params}
        action = {
            "resource": resource,
            "operation": "create",
            "body": payload
        }
        return action
        





    def updateOrganizationSmAdminsRole(self, organizationId: str, roleId: str, **kwargs):
        """
        **Update a Limited Access Role**
        https://developer.cisco.com/meraki/api-v1/#!update-organization-sm-admins-role

        - organizationId (string): Organization ID
        - roleId (string): Role ID
        - name (string): The name of the Limited Access Role
        - scope (string): The scope of the Limited Access Role
        - tags (array): The tags of the Limited Access Role
        """

        kwargs.update(locals())

        if 'scope' in kwargs:
            options = ['all_tags', 'some', 'without_all_tags', 'without_some']
            assert kwargs['scope'] in options, f'''"scope" cannot be "{kwargs['scope']}", & must be set to one of: {options}'''

        metadata = {
            'tags': ['sm', 'configure', 'admins', 'roles'],
            'operation': 'updateOrganizationSmAdminsRole'
        }
        resource = f'/organizations/{organizationId}/sm/admins/roles/{roleId}'

        body_params = ['name', 'scope', 'tags', ]
        payload = {k.strip(): v for k, v in kwargs.items() if k.strip() in body_params}
        action = {
            "resource": resource,
            "operation": "update",
            "body": payload
        }
        return action
        





    def deleteOrganizationSmAdminsRole(self, organizationId: str, roleId: str):
        """
        **Delete a Limited Access Role**
        https://developer.cisco.com/meraki/api-v1/#!delete-organization-sm-admins-role

        - organizationId (string): Organization ID
        - roleId (string): Role ID
        """

        metadata = {
            'tags': ['sm', 'configure', 'admins', 'roles'],
            'operation': 'deleteOrganizationSmAdminsRole'
        }
        resource = f'/organizations/{organizationId}/sm/admins/roles/{roleId}'

        action = {
            "resource": resource,
            "operation": "destroy",
        }
        return action
        





    def updateOrganizationSmSentryPoliciesAssignments(self, organizationId: str, items: list):
        """
        **Update an Organizations Sentry Policies using the provided list. Sentry Policies are ordered in descending order of priority (i.e. highest priority at the bottom, this is opposite the Dashboard UI). Policies not present in the request will be deleted.**
        https://developer.cisco.com/meraki/api-v1/#!update-organization-sm-sentry-policies-assignments

        - organizationId (string): Organization ID
        - items (array): Sentry Group Policies for the Organization keyed by Network Id
        """

        kwargs = locals()

        metadata = {
            'tags': ['sm', 'configure', 'sentry', 'policies', 'assignments'],
            'operation': 'updateOrganizationSmSentryPoliciesAssignments'
        }
        resource = f'/organizations/{organizationId}/sm/sentry/policies/assignments'

        body_params = ['items', ]
        payload = {k.strip(): v for k, v in kwargs.items() if k.strip() in body_params}
        action = {
            "resource": resource,
            "operation": "update",
            "body": payload
        }
        return action
        






import urllib


class ActionBatchInsight(object):
    def __init__(self):
        super(ActionBatchInsight, self).__init__()
        


    def createOrganizationInsightMonitoredMediaServer(self, organizationId: str, name: str, address: str, **kwargs):
        """
        **Add a media server to be monitored for this organization. Only valid for organizations with Meraki Insight.**
        https://developer.cisco.com/meraki/api-v1/#!create-organization-insight-monitored-media-server

        - organizationId (string): Organization ID
        - name (string): The name of the VoIP provider
        - address (string): The IP address (IPv4 only) or hostname of the media server to monitor
        - bestEffortMonitoringEnabled (boolean): Indicates that if the media server doesn't respond to ICMP pings, the nearest hop will be used in its stead.
        """

        kwargs.update(locals())

        metadata = {
            'tags': ['insight', 'configure', 'monitoredMediaServers'],
            'operation': 'createOrganizationInsightMonitoredMediaServer'
        }
        resource = f'/organizations/{organizationId}/insight/monitoredMediaServers'

        body_params = ['name', 'address', 'bestEffortMonitoringEnabled', ]
        payload = {k.strip(): v for k, v in kwargs.items() if k.strip() in body_params}
        action = {
            "resource": resource,
            "operation": "create",
            "body": payload
        }
        return action
        





    def updateOrganizationInsightMonitoredMediaServer(self, organizationId: str, monitoredMediaServerId: str, **kwargs):
        """
        **Update a monitored media server for this organization. Only valid for organizations with Meraki Insight.**
        https://developer.cisco.com/meraki/api-v1/#!update-organization-insight-monitored-media-server

        - organizationId (string): Organization ID
        - monitoredMediaServerId (string): Monitored media server ID
        - name (string): The name of the VoIP provider
        - address (string): The IP address (IPv4 only) or hostname of the media server to monitor
        - bestEffortMonitoringEnabled (boolean): Indicates that if the media server doesn't respond to ICMP pings, the nearest hop will be used in its stead.
        """

        kwargs.update(locals())

        metadata = {
            'tags': ['insight', 'configure', 'monitoredMediaServers'],
            'operation': 'updateOrganizationInsightMonitoredMediaServer'
        }
        resource = f'/organizations/{organizationId}/insight/monitoredMediaServers/{monitoredMediaServerId}'

        body_params = ['name', 'address', 'bestEffortMonitoringEnabled', ]
        payload = {k.strip(): v for k, v in kwargs.items() if k.strip() in body_params}
        action = {
            "resource": resource,
            "operation": "update",
            "body": payload
        }
        return action
        





    def deleteOrganizationInsightMonitoredMediaServer(self, organizationId: str, monitoredMediaServerId: str):
        """
        **Delete a monitored media server from this organization. Only valid for organizations with Meraki Insight.**
        https://developer.cisco.com/meraki/api-v1/#!delete-organization-insight-monitored-media-server

        - organizationId (string): Organization ID
        - monitoredMediaServerId (string): Monitored media server ID
        """

        metadata = {
            'tags': ['insight', 'configure', 'monitoredMediaServers'],
            'operation': 'deleteOrganizationInsightMonitoredMediaServer'
        }
        resource = f'/organizations/{organizationId}/insight/monitoredMediaServers/{monitoredMediaServerId}'

        action = {
            "resource": resource,
            "operation": "destroy",
        }
        return action
        






import urllib


class Camera(object):
    def __init__(self, session):
        super(Camera, self).__init__()
        self._session = session
        


    def getDeviceCameraAnalyticsLive(self, serial: str):
        """
        **Returns live state from camera analytics zones**
        https://developer.cisco.com/meraki/api-v1/#!get-device-camera-analytics-live

        - serial (string): Serial
        """

        metadata = {
            'tags': ['camera', 'monitor', 'analytics', 'live'],
            'operation': 'getDeviceCameraAnalyticsLive'
        }
        serial = urllib.parse.quote(str(serial), safe='')
        resource = f'/devices/{serial}/camera/analytics/live'

        return self._session.get(metadata, resource)
        


    def getDeviceCameraAnalyticsOverview(self, serial: str, **kwargs):
        """
        **Returns an overview of aggregate analytics data for a timespan**
        https://developer.cisco.com/meraki/api-v1/#!get-device-camera-analytics-overview

        - serial (string): Serial
        - t0 (string): The beginning of the timespan for the data. The maximum lookback period is 365 days from today.
        - t1 (string): The end of the timespan for the data. t1 can be a maximum of 7 days after t0.
        - timespan (number): The timespan for which the information will be fetched. If specifying timespan, do not specify parameters t0 and t1. The value must be in seconds and be less than or equal to 7 days. The default is 1 hour.
        - objectType (string): [optional] The object type for which analytics will be retrieved. The default object type is person. The available types are [person, vehicle].
        """

        kwargs.update(locals())

        if 'objectType' in kwargs:
            options = ['person', 'vehicle']
            assert kwargs['objectType'] in options, f'''"objectType" cannot be "{kwargs['objectType']}", & must be set to one of: {options}'''

        metadata = {
            'tags': ['camera', 'monitor', 'analytics', 'overview'],
            'operation': 'getDeviceCameraAnalyticsOverview'
        }
        serial = urllib.parse.quote(str(serial), safe='')
        resource = f'/devices/{serial}/camera/analytics/overview'

        query_params = ['t0', 't1', 'timespan', 'objectType', ]
        params = {k.strip(): v for k, v in kwargs.items() if k.strip() in query_params}

        return self._session.get(metadata, resource, params)
        


    def getDeviceCameraAnalyticsRecent(self, serial: str, **kwargs):
        """
        **Returns most recent record for analytics zones**
        https://developer.cisco.com/meraki/api-v1/#!get-device-camera-analytics-recent

        - serial (string): Serial
        - objectType (string): [optional] The object type for which analytics will be retrieved. The default object type is person. The available types are [person, vehicle].
        """

        kwargs.update(locals())

        if 'objectType' in kwargs:
            options = ['person', 'vehicle']
            assert kwargs['objectType'] in options, f'''"objectType" cannot be "{kwargs['objectType']}", & must be set to one of: {options}'''

        metadata = {
            'tags': ['camera', 'monitor', 'analytics', 'recent'],
            'operation': 'getDeviceCameraAnalyticsRecent'
        }
        serial = urllib.parse.quote(str(serial), safe='')
        resource = f'/devices/{serial}/camera/analytics/recent'

        query_params = ['objectType', ]
        params = {k.strip(): v for k, v in kwargs.items() if k.strip() in query_params}

        return self._session.get(metadata, resource, params)
        


    def getDeviceCameraAnalyticsZones(self, serial: str):
        """
        **Returns all configured analytic zones for this camera**
        https://developer.cisco.com/meraki/api-v1/#!get-device-camera-analytics-zones

        - serial (string): Serial
        """

        metadata = {
            'tags': ['camera', 'monitor', 'analytics', 'zones'],
            'operation': 'getDeviceCameraAnalyticsZones'
        }
        serial = urllib.parse.quote(str(serial), safe='')
        resource = f'/devices/{serial}/camera/analytics/zones'

        return self._session.get(metadata, resource)
        


    def getDeviceCameraAnalyticsZoneHistory(self, serial: str, zoneId: str, **kwargs):
        """
        **Return historical records for analytic zones**
        https://developer.cisco.com/meraki/api-v1/#!get-device-camera-analytics-zone-history

        - serial (string): Serial
        - zoneId (string): Zone ID
        - t0 (string): The beginning of the timespan for the data. The maximum lookback period is 365 days from today.
        - t1 (string): The end of the timespan for the data. t1 can be a maximum of 14 hours after t0.
        - timespan (number): The timespan for which the information will be fetched. If specifying timespan, do not specify parameters t0 and t1. The value must be in seconds and be less than or equal to 14 hours. The default is 1 hour.
        - resolution (integer): The time resolution in seconds for returned data. The valid resolutions are: 60. The default is 60.
        - objectType (string): [optional] The object type for which analytics will be retrieved. The default object type is person. The available types are [person, vehicle].
        """

        kwargs.update(locals())

        if 'objectType' in kwargs:
            options = ['person', 'vehicle']
            assert kwargs['objectType'] in options, f'''"objectType" cannot be "{kwargs['objectType']}", & must be set to one of: {options}'''

        metadata = {
            'tags': ['camera', 'monitor', 'analytics', 'zones', 'history'],
            'operation': 'getDeviceCameraAnalyticsZoneHistory'
        }
        serial = urllib.parse.quote(str(serial), safe='')
        zoneId = urllib.parse.quote(str(zoneId), safe='')
        resource = f'/devices/{serial}/camera/analytics/zones/{zoneId}/history'

        query_params = ['t0', 't1', 'timespan', 'resolution', 'objectType', ]
        params = {k.strip(): v for k, v in kwargs.items() if k.strip() in query_params}

        return self._session.get(metadata, resource, params)
        


    def getDeviceCameraCustomAnalytics(self, serial: str):
        """
        **Return custom analytics settings for a camera**
        https://developer.cisco.com/meraki/api-v1/#!get-device-camera-custom-analytics

        - serial (string): Serial
        """

        metadata = {
            'tags': ['camera', 'configure', 'customAnalytics'],
            'operation': 'getDeviceCameraCustomAnalytics'
        }
        serial = urllib.parse.quote(str(serial), safe='')
        resource = f'/devices/{serial}/camera/customAnalytics'

        return self._session.get(metadata, resource)
        


    def updateDeviceCameraCustomAnalytics(self, serial: str, **kwargs):
        """
        **Update custom analytics settings for a camera**
        https://developer.cisco.com/meraki/api-v1/#!update-device-camera-custom-analytics

        - serial (string): Serial
        - enabled (boolean): Enable custom analytics
        - artifactId (string): The ID of the custom analytics artifact
        - parameters (array): Parameters for the custom analytics workload
        """

        kwargs.update(locals())

        metadata = {
            'tags': ['camera', 'configure', 'customAnalytics'],
            'operation': 'updateDeviceCameraCustomAnalytics'
        }
        serial = urllib.parse.quote(str(serial), safe='')
        resource = f'/devices/{serial}/camera/customAnalytics'

        body_params = ['enabled', 'artifactId', 'parameters', ]
        payload = {k.strip(): v for k, v in kwargs.items() if k.strip() in body_params}

        return self._session.put(metadata, resource, payload)
        


    def generateDeviceCameraSnapshot(self, serial: str, **kwargs):
        """
        **Generate a snapshot of what the camera sees at the specified time and return a link to that image.**
        https://developer.cisco.com/meraki/api-v1/#!generate-device-camera-snapshot

        - serial (string): Serial
        - timestamp (string): [optional] The snapshot will be taken from this time on the camera. The timestamp is expected to be in ISO 8601 format. If no timestamp is specified, we will assume current time.
        - fullframe (boolean): [optional] If set to "true" the snapshot will be taken at full sensor resolution. This will error if used with timestamp.
        """

        kwargs.update(locals())

        metadata = {
            'tags': ['camera', 'monitor'],
            'operation': 'generateDeviceCameraSnapshot'
        }
        serial = urllib.parse.quote(str(serial), safe='')
        resource = f'/devices/{serial}/camera/generateSnapshot'

        body_params = ['timestamp', 'fullframe', ]
        payload = {k.strip(): v for k, v in kwargs.items() if k.strip() in body_params}

        return self._session.post(metadata, resource, payload)
        


    def getDeviceCameraQualityAndRetention(self, serial: str):
        """
        **Returns quality and retention settings for the given camera**
        https://developer.cisco.com/meraki/api-v1/#!get-device-camera-quality-and-retention

        - serial (string): Serial
        """

        metadata = {
            'tags': ['camera', 'configure', 'qualityAndRetention'],
            'operation': 'getDeviceCameraQualityAndRetention'
        }
        serial = urllib.parse.quote(str(serial), safe='')
        resource = f'/devices/{serial}/camera/qualityAndRetention'

        return self._session.get(metadata, resource)
        


    def updateDeviceCameraQualityAndRetention(self, serial: str, **kwargs):
        """
        **Update quality and retention settings for the given camera**
        https://developer.cisco.com/meraki/api-v1/#!update-device-camera-quality-and-retention

        - serial (string): Serial
        - profileId (string): The ID of a quality and retention profile to assign to the camera. The profile's settings will override all of the per-camera quality and retention settings. If the value of this parameter is null, any existing profile will be unassigned from the camera.
        - motionBasedRetentionEnabled (boolean): Boolean indicating if motion-based retention is enabled(true) or disabled(false) on the camera.
        - audioRecordingEnabled (boolean): Boolean indicating if audio recording is enabled(true) or disabled(false) on the camera
        - restrictedBandwidthModeEnabled (boolean): Boolean indicating if restricted bandwidth is enabled(true) or disabled(false) on the camera. This setting does not apply to MV2 cameras.
        - quality (string): Quality of the camera. Can be one of 'Standard', 'High', 'Enhanced' or 'Ultra'. Not all qualities are supported by every camera model.
        - resolution (string): Resolution of the camera. Can be one of '1280x720', '1920x1080', '1080x1080', '2112x2112', '2880x2880', '2688x1512' or '3840x2160'.Not all resolutions are supported by every camera model.
        - motionDetectorVersion (integer): The version of the motion detector that will be used by the camera. Only applies to Gen 2 cameras. Defaults to v2.
        """

        kwargs.update(locals())

        if 'quality' in kwargs:
            options = ['Enhanced', 'High', 'Standard', 'Ultra']
            assert kwargs['quality'] in options, f'''"quality" cannot be "{kwargs['quality']}", & must be set to one of: {options}'''
        if 'resolution' in kwargs:
            options = ['1080x1080', '1280x720', '1920x1080', '2112x2112', '2688x1512', '2880x2880', '3840x2160']
            assert kwargs['resolution'] in options, f'''"resolution" cannot be "{kwargs['resolution']}", & must be set to one of: {options}'''
        if 'motionDetectorVersion' in kwargs:
            options = [1, 2]
            assert kwargs['motionDetectorVersion'] in options, f'''"motionDetectorVersion" cannot be "{kwargs['motionDetectorVersion']}", & must be set to one of: {options}'''

        metadata = {
            'tags': ['camera', 'configure', 'qualityAndRetention'],
            'operation': 'updateDeviceCameraQualityAndRetention'
        }
        serial = urllib.parse.quote(str(serial), safe='')
        resource = f'/devices/{serial}/camera/qualityAndRetention'

        body_params = ['profileId', 'motionBasedRetentionEnabled', 'audioRecordingEnabled', 'restrictedBandwidthModeEnabled', 'quality', 'resolution', 'motionDetectorVersion', ]
        payload = {k.strip(): v for k, v in kwargs.items() if k.strip() in body_params}

        return self._session.put(metadata, resource, payload)
        


    def getDeviceCameraSense(self, serial: str):
        """
        **Returns sense settings for a given camera**
        https://developer.cisco.com/meraki/api-v1/#!get-device-camera-sense

        - serial (string): Serial
        """

        metadata = {
            'tags': ['camera', 'configure', 'sense'],
            'operation': 'getDeviceCameraSense'
        }
        serial = urllib.parse.quote(str(serial), safe='')
        resource = f'/devices/{serial}/camera/sense'

        return self._session.get(metadata, resource)
        


    def updateDeviceCameraSense(self, serial: str, **kwargs):
        """
        **Update sense settings for the given camera**
        https://developer.cisco.com/meraki/api-v1/#!update-device-camera-sense

        - serial (string): Serial
        - senseEnabled (boolean): Boolean indicating if sense(license) is enabled(true) or disabled(false) on the camera
        - mqttBrokerId (string): The ID of the MQTT broker to be enabled on the camera. A value of null will disable MQTT on the camera
        - audioDetection (object): The details of the audio detection config.
        - detectionModelId (string): The ID of the object detection model
        """

        kwargs.update(locals())

        metadata = {
            'tags': ['camera', 'configure', 'sense'],
            'operation': 'updateDeviceCameraSense'
        }
        serial = urllib.parse.quote(str(serial), safe='')
        resource = f'/devices/{serial}/camera/sense'

        body_params = ['senseEnabled', 'mqttBrokerId', 'audioDetection', 'detectionModelId', ]
        payload = {k.strip(): v for k, v in kwargs.items() if k.strip() in body_params}

        return self._session.put(metadata, resource, payload)
        


    def getDeviceCameraSenseObjectDetectionModels(self, serial: str):
        """
        **Returns the MV Sense object detection model list for the given camera**
        https://developer.cisco.com/meraki/api-v1/#!get-device-camera-sense-object-detection-models

        - serial (string): Serial
        """

        metadata = {
            'tags': ['camera', 'configure', 'sense', 'objectDetectionModels'],
            'operation': 'getDeviceCameraSenseObjectDetectionModels'
        }
        serial = urllib.parse.quote(str(serial), safe='')
        resource = f'/devices/{serial}/camera/sense/objectDetectionModels'

        return self._session.get(metadata, resource)
        


    def getDeviceCameraVideoSettings(self, serial: str):
        """
        **Returns video settings for the given camera**
        https://developer.cisco.com/meraki/api-v1/#!get-device-camera-video-settings

        - serial (string): Serial
        """

        metadata = {
            'tags': ['camera', 'configure', 'video', 'settings'],
            'operation': 'getDeviceCameraVideoSettings'
        }
        serial = urllib.parse.quote(str(serial), safe='')
        resource = f'/devices/{serial}/camera/video/settings'

        return self._session.get(metadata, resource)
        


    def updateDeviceCameraVideoSettings(self, serial: str, **kwargs):
        """
        **Update video settings for the given camera**
        https://developer.cisco.com/meraki/api-v1/#!update-device-camera-video-settings

        - serial (string): Serial
        - externalRtspEnabled (boolean): Boolean indicating if external rtsp stream is exposed
        """

        kwargs.update(locals())

        metadata = {
            'tags': ['camera', 'configure', 'video', 'settings'],
            'operation': 'updateDeviceCameraVideoSettings'
        }
        serial = urllib.parse.quote(str(serial), safe='')
        resource = f'/devices/{serial}/camera/video/settings'

        body_params = ['externalRtspEnabled', ]
        payload = {k.strip(): v for k, v in kwargs.items() if k.strip() in body_params}

        return self._session.put(metadata, resource, payload)
        


    def getDeviceCameraVideoLink(self, serial: str, **kwargs):
        """
        **Returns video link to the specified camera**
        https://developer.cisco.com/meraki/api-v1/#!get-device-camera-video-link

        - serial (string): Serial
        - timestamp (string): [optional] The video link will start at this time. The timestamp should be a string in ISO8601 format. If no timestamp is specified, we will assume current time.
        """

        kwargs.update(locals())

        metadata = {
            'tags': ['camera', 'configure', 'videoLink'],
            'operation': 'getDeviceCameraVideoLink'
        }
        serial = urllib.parse.quote(str(serial), safe='')
        resource = f'/devices/{serial}/camera/videoLink'

        query_params = ['timestamp', ]
        params = {k.strip(): v for k, v in kwargs.items() if k.strip() in query_params}

        return self._session.get(metadata, resource, params)
        


    def getDeviceCameraWirelessProfiles(self, serial: str):
        """
        **Returns wireless profile assigned to the given camera**
        https://developer.cisco.com/meraki/api-v1/#!get-device-camera-wireless-profiles

        - serial (string): Serial
        """

        metadata = {
            'tags': ['camera', 'configure', 'wirelessProfiles'],
            'operation': 'getDeviceCameraWirelessProfiles'
        }
        serial = urllib.parse.quote(str(serial), safe='')
        resource = f'/devices/{serial}/camera/wirelessProfiles'

        return self._session.get(metadata, resource)
        


    def updateDeviceCameraWirelessProfiles(self, serial: str, ids: dict):
        """
        **Assign wireless profiles to the given camera**
        https://developer.cisco.com/meraki/api-v1/#!update-device-camera-wireless-profiles

        - serial (string): Serial
        - ids (object): The ids of the wireless profile to assign to the given camera
        """

        kwargs = locals()

        metadata = {
            'tags': ['camera', 'configure', 'wirelessProfiles'],
            'operation': 'updateDeviceCameraWirelessProfiles'
        }
        serial = urllib.parse.quote(str(serial), safe='')
        resource = f'/devices/{serial}/camera/wirelessProfiles'

        body_params = ['ids', ]
        payload = {k.strip(): v for k, v in kwargs.items() if k.strip() in body_params}

        return self._session.put(metadata, resource, payload)
        


    def getNetworkCameraQualityRetentionProfiles(self, networkId: str):
        """
        **List the quality retention profiles for this network**
        https://developer.cisco.com/meraki/api-v1/#!get-network-camera-quality-retention-profiles

        - networkId (string): Network ID
        """

        metadata = {
            'tags': ['camera', 'configure', 'qualityRetentionProfiles'],
            'operation': 'getNetworkCameraQualityRetentionProfiles'
        }
        networkId = urllib.parse.quote(str(networkId), safe='')
        resource = f'/networks/{networkId}/camera/qualityRetentionProfiles'

        return self._session.get(metadata, resource)
        


    def createNetworkCameraQualityRetentionProfile(self, networkId: str, name: str, **kwargs):
        """
        **Creates new quality retention profile for this network.**
        https://developer.cisco.com/meraki/api-v1/#!create-network-camera-quality-retention-profile

        - networkId (string): Network ID
        - name (string): The name of the new profile. Must be unique. This parameter is required.
        - motionBasedRetentionEnabled (boolean): Deletes footage older than 3 days in which no motion was detected. Can be either true or false. Defaults to false. This setting does not apply to MV2 cameras.
        - restrictedBandwidthModeEnabled (boolean): Disable features that require additional bandwidth such as Motion Recap. Can be either true or false. Defaults to false. This setting does not apply to MV2 cameras.
        - audioRecordingEnabled (boolean): Whether or not to record audio. Can be either true or false. Defaults to false.
        - cloudArchiveEnabled (boolean): Create redundant video backup using Cloud Archive. Can be either true or false. Defaults to false.
        - motionDetectorVersion (integer): The version of the motion detector that will be used by the camera. Only applies to Gen 2 cameras. Defaults to v2.
        - smartRetention (object): Smart Retention records footage in two qualities and intelligently retains higher quality when motion, people or vehicles are detected.
        - scheduleId (string): Schedule for which this camera will record video, or 'null' to always record.
        - maxRetentionDays (integer): The maximum number of days for which the data will be stored, or 'null' to keep data until storage space runs out. If the former, it can be in the range of one to ninety days.
        - videoSettings (object): Video quality and resolution settings for all the camera models.
        """

        kwargs.update(locals())

        metadata = {
            'tags': ['camera', 'configure', 'qualityRetentionProfiles'],
            'operation': 'createNetworkCameraQualityRetentionProfile'
        }
        networkId = urllib.parse.quote(str(networkId), safe='')
        resource = f'/networks/{networkId}/camera/qualityRetentionProfiles'

        body_params = ['name', 'motionBasedRetentionEnabled', 'restrictedBandwidthModeEnabled', 'audioRecordingEnabled', 'cloudArchiveEnabled', 'motionDetectorVersion', 'smartRetention', 'scheduleId', 'maxRetentionDays', 'videoSettings', ]
        payload = {k.strip(): v for k, v in kwargs.items() if k.strip() in body_params}

        return self._session.post(metadata, resource, payload)
        


    def getNetworkCameraQualityRetentionProfile(self, networkId: str, qualityRetentionProfileId: str):
        """
        **Retrieve a single quality retention profile**
        https://developer.cisco.com/meraki/api-v1/#!get-network-camera-quality-retention-profile

        - networkId (string): Network ID
        - qualityRetentionProfileId (string): Quality retention profile ID
        """

        metadata = {
            'tags': ['camera', 'configure', 'qualityRetentionProfiles'],
            'operation': 'getNetworkCameraQualityRetentionProfile'
        }
        networkId = urllib.parse.quote(str(networkId), safe='')
        qualityRetentionProfileId = urllib.parse.quote(str(qualityRetentionProfileId), safe='')
        resource = f'/networks/{networkId}/camera/qualityRetentionProfiles/{qualityRetentionProfileId}'

        return self._session.get(metadata, resource)
        


    def updateNetworkCameraQualityRetentionProfile(self, networkId: str, qualityRetentionProfileId: str, **kwargs):
        """
        **Update an existing quality retention profile for this network.**
        https://developer.cisco.com/meraki/api-v1/#!update-network-camera-quality-retention-profile

        - networkId (string): Network ID
        - qualityRetentionProfileId (string): Quality retention profile ID
        - name (string): The name of the new profile. Must be unique.
        - motionBasedRetentionEnabled (boolean): Deletes footage older than 3 days in which no motion was detected. Can be either true or false. Defaults to false. This setting does not apply to MV2 cameras.
        - restrictedBandwidthModeEnabled (boolean): Disable features that require additional bandwidth such as Motion Recap. Can be either true or false. Defaults to false. This setting does not apply to MV2 cameras.
        - audioRecordingEnabled (boolean): Whether or not to record audio. Can be either true or false. Defaults to false.
        - cloudArchiveEnabled (boolean): Create redundant video backup using Cloud Archive. Can be either true or false. Defaults to false.
        - motionDetectorVersion (integer): The version of the motion detector that will be used by the camera. Only applies to Gen 2 cameras. Defaults to v2.
        - smartRetention (object): Smart Retention records footage in two qualities and intelligently retains higher quality when motion, people or vehicles are detected.
        - scheduleId (string): Schedule for which this camera will record video, or 'null' to always record.
        - maxRetentionDays (integer): The maximum number of days for which the data will be stored, or 'null' to keep data until storage space runs out. If the former, it can be in the range of one to ninety days.
        - videoSettings (object): Video quality and resolution settings for all the camera models.
        """

        kwargs.update(locals())

        metadata = {
            'tags': ['camera', 'configure', 'qualityRetentionProfiles'],
            'operation': 'updateNetworkCameraQualityRetentionProfile'
        }
        networkId = urllib.parse.quote(str(networkId), safe='')
        qualityRetentionProfileId = urllib.parse.quote(str(qualityRetentionProfileId), safe='')
        resource = f'/networks/{networkId}/camera/qualityRetentionProfiles/{qualityRetentionProfileId}'

        body_params = ['name', 'motionBasedRetentionEnabled', 'restrictedBandwidthModeEnabled', 'audioRecordingEnabled', 'cloudArchiveEnabled', 'motionDetectorVersion', 'smartRetention', 'scheduleId', 'maxRetentionDays', 'videoSettings', ]
        payload = {k.strip(): v for k, v in kwargs.items() if k.strip() in body_params}

        return self._session.put(metadata, resource, payload)
        


    def deleteNetworkCameraQualityRetentionProfile(self, networkId: str, qualityRetentionProfileId: str):
        """
        **Delete an existing quality retention profile for this network.**
        https://developer.cisco.com/meraki/api-v1/#!delete-network-camera-quality-retention-profile

        - networkId (string): Network ID
        - qualityRetentionProfileId (string): Quality retention profile ID
        """

        metadata = {
            'tags': ['camera', 'configure', 'qualityRetentionProfiles'],
            'operation': 'deleteNetworkCameraQualityRetentionProfile'
        }
        networkId = urllib.parse.quote(str(networkId), safe='')
        qualityRetentionProfileId = urllib.parse.quote(str(qualityRetentionProfileId), safe='')
        resource = f'/networks/{networkId}/camera/qualityRetentionProfiles/{qualityRetentionProfileId}'

        return self._session.delete(metadata, resource)
        


    def getNetworkCameraSchedules(self, networkId: str):
        """
        **Returns a list of all camera recording schedules.**
        https://developer.cisco.com/meraki/api-v1/#!get-network-camera-schedules

        - networkId (string): Network ID
        """

        metadata = {
            'tags': ['camera', 'configure', 'schedules'],
            'operation': 'getNetworkCameraSchedules'
        }
        networkId = urllib.parse.quote(str(networkId), safe='')
        resource = f'/networks/{networkId}/camera/schedules'

        return self._session.get(metadata, resource)
        


    def createNetworkCameraWirelessProfile(self, networkId: str, name: str, ssid: dict, **kwargs):
        """
        **Creates a new camera wireless profile for this network.**
        https://developer.cisco.com/meraki/api-v1/#!create-network-camera-wireless-profile

        - networkId (string): Network ID
        - name (string): The name of the camera wireless profile. This parameter is required.
        - ssid (object): The details of the SSID config.
        - identity (object): The identity of the wireless profile. Required for creating wireless profiles in 8021x-radius auth mode.
        """

        kwargs.update(locals())

        metadata = {
            'tags': ['camera', 'configure', 'wirelessProfiles'],
            'operation': 'createNetworkCameraWirelessProfile'
        }
        networkId = urllib.parse.quote(str(networkId), safe='')
        resource = f'/networks/{networkId}/camera/wirelessProfiles'

        body_params = ['name', 'ssid', 'identity', ]
        payload = {k.strip(): v for k, v in kwargs.items() if k.strip() in body_params}

        return self._session.post(metadata, resource, payload)
        


    def getNetworkCameraWirelessProfiles(self, networkId: str):
        """
        **List the camera wireless profiles for this network.**
        https://developer.cisco.com/meraki/api-v1/#!get-network-camera-wireless-profiles

        - networkId (string): Network ID
        """

        metadata = {
            'tags': ['camera', 'configure', 'wirelessProfiles'],
            'operation': 'getNetworkCameraWirelessProfiles'
        }
        networkId = urllib.parse.quote(str(networkId), safe='')
        resource = f'/networks/{networkId}/camera/wirelessProfiles'

        return self._session.get(metadata, resource)
        


    def getNetworkCameraWirelessProfile(self, networkId: str, wirelessProfileId: str):
        """
        **Retrieve a single camera wireless profile.**
        https://developer.cisco.com/meraki/api-v1/#!get-network-camera-wireless-profile

        - networkId (string): Network ID
        - wirelessProfileId (string): Wireless profile ID
        """

        metadata = {
            'tags': ['camera', 'configure', 'wirelessProfiles'],
            'operation': 'getNetworkCameraWirelessProfile'
        }
        networkId = urllib.parse.quote(str(networkId), safe='')
        wirelessProfileId = urllib.parse.quote(str(wirelessProfileId), safe='')
        resource = f'/networks/{networkId}/camera/wirelessProfiles/{wirelessProfileId}'

        return self._session.get(metadata, resource)
        


    def updateNetworkCameraWirelessProfile(self, networkId: str, wirelessProfileId: str, **kwargs):
        """
        **Update an existing camera wireless profile in this network.**
        https://developer.cisco.com/meraki/api-v1/#!update-network-camera-wireless-profile

        - networkId (string): Network ID
        - wirelessProfileId (string): Wireless profile ID
        - name (string): The name of the camera wireless profile.
        - ssid (object): The details of the SSID config.
        - identity (object): The identity of the wireless profile. Required for creating wireless profiles in 8021x-radius auth mode.
        """

        kwargs.update(locals())

        metadata = {
            'tags': ['camera', 'configure', 'wirelessProfiles'],
            'operation': 'updateNetworkCameraWirelessProfile'
        }
        networkId = urllib.parse.quote(str(networkId), safe='')
        wirelessProfileId = urllib.parse.quote(str(wirelessProfileId), safe='')
        resource = f'/networks/{networkId}/camera/wirelessProfiles/{wirelessProfileId}'

        body_params = ['name', 'ssid', 'identity', ]
        payload = {k.strip(): v for k, v in kwargs.items() if k.strip() in body_params}

        return self._session.put(metadata, resource, payload)
        


    def deleteNetworkCameraWirelessProfile(self, networkId: str, wirelessProfileId: str):
        """
        **Delete an existing camera wireless profile for this network.**
        https://developer.cisco.com/meraki/api-v1/#!delete-network-camera-wireless-profile

        - networkId (string): Network ID
        - wirelessProfileId (string): Wireless profile ID
        """

        metadata = {
            'tags': ['camera', 'configure', 'wirelessProfiles'],
            'operation': 'deleteNetworkCameraWirelessProfile'
        }
        networkId = urllib.parse.quote(str(networkId), safe='')
        wirelessProfileId = urllib.parse.quote(str(wirelessProfileId), safe='')
        resource = f'/networks/{networkId}/camera/wirelessProfiles/{wirelessProfileId}'

        return self._session.delete(metadata, resource)
        


    def getOrganizationCameraBoundariesAreasByDevice(self, organizationId: str, **kwargs):
        """
        **Returns all configured area boundaries of cameras**
        https://developer.cisco.com/meraki/api-v1/#!get-organization-camera-boundaries-areas-by-device

        - organizationId (string): Organization ID
        - serials (array): A list of serial numbers. The returned cameras will be filtered to only include these serials.
        """

        kwargs.update(locals())

        metadata = {
            'tags': ['camera', 'configure', 'boundaries', 'areas', 'byDevice'],
            'operation': 'getOrganizationCameraBoundariesAreasByDevice'
        }
        organizationId = urllib.parse.quote(str(organizationId), safe='')
        resource = f'/organizations/{organizationId}/camera/boundaries/areas/byDevice'

        query_params = ['serials', ]
        params = {k.strip(): v for k, v in kwargs.items() if k.strip() in query_params}

        array_params = ['serials', ]
        for k, v in kwargs.items():
            if k.strip() in array_params:
                params[f'{k.strip()}[]'] = kwargs[f'{k}']
                params.pop(k.strip())

        return self._session.get(metadata, resource, params)
        


    def getOrganizationCameraBoundariesLinesByDevice(self, organizationId: str, **kwargs):
        """
        **Returns all configured crossingline boundaries of cameras**
        https://developer.cisco.com/meraki/api-v1/#!get-organization-camera-boundaries-lines-by-device

        - organizationId (string): Organization ID
        - serials (array): A list of serial numbers. The returned cameras will be filtered to only include these serials.
        """

        kwargs.update(locals())

        metadata = {
            'tags': ['camera', 'configure', 'boundaries', 'lines', 'byDevice'],
            'operation': 'getOrganizationCameraBoundariesLinesByDevice'
        }
        organizationId = urllib.parse.quote(str(organizationId), safe='')
        resource = f'/organizations/{organizationId}/camera/boundaries/lines/byDevice'

        query_params = ['serials', ]
        params = {k.strip(): v for k, v in kwargs.items() if k.strip() in query_params}

        array_params = ['serials', ]
        for k, v in kwargs.items():
            if k.strip() in array_params:
                params[f'{k.strip()}[]'] = kwargs[f'{k}']
                params.pop(k.strip())

        return self._session.get(metadata, resource, params)
        


    def getOrganizationCameraCustomAnalyticsArtifacts(self, organizationId: str):
        """
        **List Custom Analytics Artifacts**
        https://developer.cisco.com/meraki/api-v1/#!get-organization-camera-custom-analytics-artifacts

        - organizationId (string): Organization ID
        """

        metadata = {
            'tags': ['camera', 'configure', 'customAnalytics', 'artifacts'],
            'operation': 'getOrganizationCameraCustomAnalyticsArtifacts'
        }
        organizationId = urllib.parse.quote(str(organizationId), safe='')
        resource = f'/organizations/{organizationId}/camera/customAnalytics/artifacts'

        return self._session.get(metadata, resource)
        


    def createOrganizationCameraCustomAnalyticsArtifact(self, organizationId: str, **kwargs):
        """
        **Create custom analytics artifact**
        https://developer.cisco.com/meraki/api-v1/#!create-organization-camera-custom-analytics-artifact

        - organizationId (string): Organization ID
        - name (string): Unique name of the artifact
        """

        kwargs.update(locals())

        metadata = {
            'tags': ['camera', 'configure', 'customAnalytics', 'artifacts'],
            'operation': 'createOrganizationCameraCustomAnalyticsArtifact'
        }
        organizationId = urllib.parse.quote(str(organizationId), safe='')
        resource = f'/organizations/{organizationId}/camera/customAnalytics/artifacts'

        body_params = ['name', ]
        payload = {k.strip(): v for k, v in kwargs.items() if k.strip() in body_params}

        return self._session.post(metadata, resource, payload)
        


    def getOrganizationCameraCustomAnalyticsArtifact(self, organizationId: str, artifactId: str):
        """
        **Get Custom Analytics Artifact**
        https://developer.cisco.com/meraki/api-v1/#!get-organization-camera-custom-analytics-artifact

        - organizationId (string): Organization ID
        - artifactId (string): Artifact ID
        """

        metadata = {
            'tags': ['camera', 'configure', 'customAnalytics', 'artifacts'],
            'operation': 'getOrganizationCameraCustomAnalyticsArtifact'
        }
        organizationId = urllib.parse.quote(str(organizationId), safe='')
        artifactId = urllib.parse.quote(str(artifactId), safe='')
        resource = f'/organizations/{organizationId}/camera/customAnalytics/artifacts/{artifactId}'

        return self._session.get(metadata, resource)
        


    def deleteOrganizationCameraCustomAnalyticsArtifact(self, organizationId: str, artifactId: str):
        """
        **Delete Custom Analytics Artifact**
        https://developer.cisco.com/meraki/api-v1/#!delete-organization-camera-custom-analytics-artifact

        - organizationId (string): Organization ID
        - artifactId (string): Artifact ID
        """

        metadata = {
            'tags': ['camera', 'configure', 'customAnalytics', 'artifacts'],
            'operation': 'deleteOrganizationCameraCustomAnalyticsArtifact'
        }
        organizationId = urllib.parse.quote(str(organizationId), safe='')
        artifactId = urllib.parse.quote(str(artifactId), safe='')
        resource = f'/organizations/{organizationId}/camera/customAnalytics/artifacts/{artifactId}'

        return self._session.delete(metadata, resource)
        


    def getOrganizationCameraDetectionsHistoryByBoundaryByInterval(self, organizationId: str, boundaryIds: list, total_pages=1, direction='next', **kwargs):
        """
        **Returns analytics data for timespans**
        https://developer.cisco.com/meraki/api-v1/#!get-organization-camera-detections-history-by-boundary-by-interval

        - organizationId (string): Organization ID
        - boundaryIds (array): A list of boundary ids. The returned cameras will be filtered to only include these ids.
        - total_pages (integer or string): use with perPage to get total results up to total_pages*perPage; -1 or "all" for all pages
        - direction (string): direction to paginate, either "next" (default) or "prev" page
        - duration (integer): The minimum time, in seconds, that the person or car remains in the area to be counted. Defaults to boundary configuration or 60.
        - perPage (integer): The number of entries per page returned. Acceptable range is 1 - 1000. Defaults to 1000.
        - boundaryTypes (array): The detection types. Defaults to 'person'.
        """

        kwargs.update(locals())

        metadata = {
            'tags': ['camera', 'configure', 'detections', 'history', 'byBoundary', 'byInterval'],
            'operation': 'getOrganizationCameraDetectionsHistoryByBoundaryByInterval'
        }
        organizationId = urllib.parse.quote(str(organizationId), safe='')
        resource = f'/organizations/{organizationId}/camera/detections/history/byBoundary/byInterval'

        query_params = ['boundaryIds', 'duration', 'perPage', 'boundaryTypes', ]
        params = {k.strip(): v for k, v in kwargs.items() if k.strip() in query_params}

        array_params = ['boundaryIds', 'boundaryTypes', ]
        for k, v in kwargs.items():
            if k.strip() in array_params:
                params[f'{k.strip()}[]'] = kwargs[f'{k}']
                params.pop(k.strip())

        return self._session.get_pages(metadata, resource, params, total_pages, direction)
        


    def getOrganizationCameraOnboardingStatuses(self, organizationId: str, **kwargs):
        """
        **Fetch onboarding status of cameras**
        https://developer.cisco.com/meraki/api-v1/#!get-organization-camera-onboarding-statuses

        - organizationId (string): Organization ID
        - serials (array): A list of serial numbers. The returned cameras will be filtered to only include these serials.
        - networkIds (array): A list of network IDs. The returned cameras will be filtered to only include these networks.
        """

        kwargs.update(locals())

        metadata = {
            'tags': ['camera', 'configure', 'onboarding', 'statuses'],
            'operation': 'getOrganizationCameraOnboardingStatuses'
        }
        organizationId = urllib.parse.quote(str(organizationId), safe='')
        resource = f'/organizations/{organizationId}/camera/onboarding/statuses'

        query_params = ['serials', 'networkIds', ]
        params = {k.strip(): v for k, v in kwargs.items() if k.strip() in query_params}

        array_params = ['serials', 'networkIds', ]
        for k, v in kwargs.items():
            if k.strip() in array_params:
                params[f'{k.strip()}[]'] = kwargs[f'{k}']
                params.pop(k.strip())

        return self._session.get(metadata, resource, params)
        


    def updateOrganizationCameraOnboardingStatuses(self, organizationId: str, **kwargs):
        """
        **Notify that credential handoff to camera has completed**
        https://developer.cisco.com/meraki/api-v1/#!update-organization-camera-onboarding-statuses

        - organizationId (string): Organization ID
        - serial (string): Serial of camera
        - wirelessCredentialsSent (boolean): Note whether credentials were sent successfully
        """

        kwargs.update(locals())

        metadata = {
            'tags': ['camera', 'configure', 'onboarding', 'statuses'],
            'operation': 'updateOrganizationCameraOnboardingStatuses'
        }
        organizationId = urllib.parse.quote(str(organizationId), safe='')
        resource = f'/organizations/{organizationId}/camera/onboarding/statuses'

        body_params = ['serial', 'wirelessCredentialsSent', ]
        payload = {k.strip(): v for k, v in kwargs.items() if k.strip() in body_params}

        return self._session.put(metadata, resource, payload)
        


    def getOrganizationCameraPermissions(self, organizationId: str):
        """
        **List the permissions scopes for this organization**
        https://developer.cisco.com/meraki/api-v1/#!get-organization-camera-permissions

        - organizationId (string): Organization ID
        """

        metadata = {
            'tags': ['camera', 'configure', 'permissions'],
            'operation': 'getOrganizationCameraPermissions'
        }
        organizationId = urllib.parse.quote(str(organizationId), safe='')
        resource = f'/organizations/{organizationId}/camera/permissions'

        return self._session.get(metadata, resource)
        


    def getOrganizationCameraPermission(self, organizationId: str, permissionScopeId: str):
        """
        **Retrieve a single permission scope**
        https://developer.cisco.com/meraki/api-v1/#!get-organization-camera-permission

        - organizationId (string): Organization ID
        - permissionScopeId (string): Permission scope ID
        """

        metadata = {
            'tags': ['camera', 'configure', 'permissions'],
            'operation': 'getOrganizationCameraPermission'
        }
        organizationId = urllib.parse.quote(str(organizationId), safe='')
        permissionScopeId = urllib.parse.quote(str(permissionScopeId), safe='')
        resource = f'/organizations/{organizationId}/camera/permissions/{permissionScopeId}'

        return self._session.get(metadata, resource)
        


    def getOrganizationCameraRoles(self, organizationId: str):
        """
        **List all the roles in this organization**
        https://developer.cisco.com/meraki/api-v1/#!get-organization-camera-roles

        - organizationId (string): Organization ID
        """

        metadata = {
            'tags': ['camera', 'configure', 'roles'],
            'operation': 'getOrganizationCameraRoles'
        }
        organizationId = urllib.parse.quote(str(organizationId), safe='')
        resource = f'/organizations/{organizationId}/camera/roles'

        return self._session.get(metadata, resource)
        


    def createOrganizationCameraRole(self, organizationId: str, name: str, **kwargs):
        """
        **Creates new role for this organization.**
        https://developer.cisco.com/meraki/api-v1/#!create-organization-camera-role

        - organizationId (string): Organization ID
        - name (string): The name of the new role. Must be unique. This parameter is required.
        - appliedOnDevices (array): Device tag on which this specified permission is applied.
        - appliedOnNetworks (array): Network tag on which this specified permission is applied.
        - appliedOrgWide (array): Permissions to be applied org wide.
        """

        kwargs.update(locals())

        metadata = {
            'tags': ['camera', 'configure', 'roles'],
            'operation': 'createOrganizationCameraRole'
        }
        organizationId = urllib.parse.quote(str(organizationId), safe='')
        resource = f'/organizations/{organizationId}/camera/roles'

        body_params = ['name', 'appliedOnDevices', 'appliedOnNetworks', 'appliedOrgWide', ]
        payload = {k.strip(): v for k, v in kwargs.items() if k.strip() in body_params}

        return self._session.post(metadata, resource, payload)
        


    def getOrganizationCameraRole(self, organizationId: str, roleId: str):
        """
        **Retrieve a single role.**
        https://developer.cisco.com/meraki/api-v1/#!get-organization-camera-role

        - organizationId (string): Organization ID
        - roleId (string): Role ID
        """

        metadata = {
            'tags': ['camera', 'configure', 'roles'],
            'operation': 'getOrganizationCameraRole'
        }
        organizationId = urllib.parse.quote(str(organizationId), safe='')
        roleId = urllib.parse.quote(str(roleId), safe='')
        resource = f'/organizations/{organizationId}/camera/roles/{roleId}'

        return self._session.get(metadata, resource)
        


    def deleteOrganizationCameraRole(self, organizationId: str, roleId: str):
        """
        **Delete an existing role for this organization.**
        https://developer.cisco.com/meraki/api-v1/#!delete-organization-camera-role

        - organizationId (string): Organization ID
        - roleId (string): Role ID
        """

        metadata = {
            'tags': ['camera', 'configure', 'roles'],
            'operation': 'deleteOrganizationCameraRole'
        }
        organizationId = urllib.parse.quote(str(organizationId), safe='')
        roleId = urllib.parse.quote(str(roleId), safe='')
        resource = f'/organizations/{organizationId}/camera/roles/{roleId}'

        return self._session.delete(metadata, resource)
        


    def updateOrganizationCameraRole(self, organizationId: str, roleId: str, **kwargs):
        """
        **Update an existing role in this organization.**
        https://developer.cisco.com/meraki/api-v1/#!update-organization-camera-role

        - organizationId (string): Organization ID
        - roleId (string): Role ID
        - name (string): The name of the new role. Must be unique.
        - appliedOnDevices (array): Device tag on which this specified permission is applied.
        - appliedOnNetworks (array): Network tag on which this specified permission is applied.
        - appliedOrgWide (array): Permissions to be applied org wide.
        """

        kwargs.update(locals())

        metadata = {
            'tags': ['camera', 'configure', 'roles'],
            'operation': 'updateOrganizationCameraRole'
        }
        organizationId = urllib.parse.quote(str(organizationId), safe='')
        roleId = urllib.parse.quote(str(roleId), safe='')
        resource = f'/organizations/{organizationId}/camera/roles/{roleId}'

        body_params = ['name', 'appliedOnDevices', 'appliedOnNetworks', 'appliedOrgWide', ]
        payload = {k.strip(): v for k, v in kwargs.items() if k.strip() in body_params}

        return self._session.put(metadata, resource, payload)
        



import urllib


class ActionBatchSensor(object):
    def __init__(self):
        super(ActionBatchSensor, self).__init__()
        


    def createDeviceSensorCommand(self, serial: str, operation: str):
        """
        **Sends a command to a sensor**
        https://developer.cisco.com/meraki/api-v1/#!create-device-sensor-command

        - serial (string): Serial
        - operation (string): Operation to run on the sensor. 'enableDownstreamPower', 'disableDownstreamPower', and 'cycleDownstreamPower' turn power on/off to the device that is connected downstream of an MT40 power monitor. 'refreshData' causes an MT15 or MT40 device to upload its latest readings so that they are immediately available in the Dashboard API.
        """

        kwargs = locals()

        if 'operation' in kwargs:
            options = ['cycleDownstreamPower', 'disableDownstreamPower', 'enableDownstreamPower', 'refreshData']
            assert kwargs['operation'] in options, f'''"operation" cannot be "{kwargs['operation']}", & must be set to one of: {options}'''

        metadata = {
            'tags': ['sensor', 'configure', 'commands'],
            'operation': 'createDeviceSensorCommand'
        }
        resource = f'/devices/{serial}/sensor/commands'

        body_params = ['operation', ]
        payload = {k.strip(): v for k, v in kwargs.items() if k.strip() in body_params}
        action = {
            "resource": resource,
            "operation": "create",
            "body": payload
        }
        return action
        





    def updateDeviceSensorRelationships(self, serial: str, **kwargs):
        """
        **Assign one or more sensor roles to a given sensor or camera device.**
        https://developer.cisco.com/meraki/api-v1/#!update-device-sensor-relationships

        - serial (string): Serial
        - livestream (object): A role defined between an MT sensor and an MV camera that adds the camera's livestream to the sensor's details page. Snapshots from the camera will also appear in alert notifications that the sensor triggers.
        """

        kwargs.update(locals())

        metadata = {
            'tags': ['sensor', 'configure', 'relationships'],
            'operation': 'updateDeviceSensorRelationships'
        }
        resource = f'/devices/{serial}/sensor/relationships'

        body_params = ['livestream', ]
        payload = {k.strip(): v for k, v in kwargs.items() if k.strip() in body_params}
        action = {
            "resource": resource,
            "operation": "update",
            "body": payload
        }
        return action
        





    def createNetworkSensorAlertsProfile(self, networkId: str, name: str, conditions: list, **kwargs):
        """
        **Creates a sensor alert profile for a network.**
        https://developer.cisco.com/meraki/api-v1/#!create-network-sensor-alerts-profile

        - networkId (string): Network ID
        - name (string): Name of the sensor alert profile.
        - conditions (array): List of conditions that will cause the profile to send an alert.
        - schedule (object): The sensor schedule to use with the alert profile.
        - recipients (object): List of recipients that will receive the alert.
        - serials (array): List of device serials assigned to this sensor alert profile.
        - includeSensorUrl (boolean): Include dashboard link to sensor in messages (default: true).
        - message (string): A custom message that will appear in email and text message alerts.
        """

        kwargs.update(locals())

        metadata = {
            'tags': ['sensor', 'configure', 'alerts', 'profiles'],
            'operation': 'createNetworkSensorAlertsProfile'
        }
        resource = f'/networks/{networkId}/sensor/alerts/profiles'

        body_params = ['name', 'schedule', 'conditions', 'recipients', 'serials', 'includeSensorUrl', 'message', ]
        payload = {k.strip(): v for k, v in kwargs.items() if k.strip() in body_params}
        action = {
            "resource": resource,
            "operation": "create",
            "body": payload
        }
        return action
        





    def updateNetworkSensorAlertsProfile(self, networkId: str, id: str, **kwargs):
        """
        **Updates a sensor alert profile for a network.**
        https://developer.cisco.com/meraki/api-v1/#!update-network-sensor-alerts-profile

        - networkId (string): Network ID
        - id (string): ID
        - name (string): Name of the sensor alert profile.
        - schedule (object): The sensor schedule to use with the alert profile.
        - conditions (array): List of conditions that will cause the profile to send an alert.
        - recipients (object): List of recipients that will receive the alert.
        - serials (array): List of device serials assigned to this sensor alert profile.
        - includeSensorUrl (boolean): Include dashboard link to sensor in messages (default: true).
        - message (string): A custom message that will appear in email and text message alerts.
        """

        kwargs.update(locals())

        metadata = {
            'tags': ['sensor', 'configure', 'alerts', 'profiles'],
            'operation': 'updateNetworkSensorAlertsProfile'
        }
        resource = f'/networks/{networkId}/sensor/alerts/profiles/{id}'

        body_params = ['name', 'schedule', 'conditions', 'recipients', 'serials', 'includeSensorUrl', 'message', ]
        payload = {k.strip(): v for k, v in kwargs.items() if k.strip() in body_params}
        action = {
            "resource": resource,
            "operation": "update",
            "body": payload
        }
        return action
        





    def deleteNetworkSensorAlertsProfile(self, networkId: str, id: str):
        """
        **Deletes a sensor alert profile from a network.**
        https://developer.cisco.com/meraki/api-v1/#!delete-network-sensor-alerts-profile

        - networkId (string): Network ID
        - id (string): ID
        """

        metadata = {
            'tags': ['sensor', 'configure', 'alerts', 'profiles'],
            'operation': 'deleteNetworkSensorAlertsProfile'
        }
        resource = f'/networks/{networkId}/sensor/alerts/profiles/{id}'

        action = {
            "resource": resource,
            "operation": "destroy",
        }
        return action
        





    def updateNetworkSensorMqttBroker(self, networkId: str, mqttBrokerId: str, enabled: bool):
        """
        **Update the sensor settings of an MQTT broker. To update the broker itself, use /networks/{networkId}/mqttBrokers/{mqttBrokerId}.**
        https://developer.cisco.com/meraki/api-v1/#!update-network-sensor-mqtt-broker

        - networkId (string): Network ID
        - mqttBrokerId (string): Mqtt broker ID
        - enabled (boolean): Set to true to enable MQTT broker for sensor network
        """

        kwargs = locals()

        metadata = {
            'tags': ['sensor', 'configure', 'mqttBrokers'],
            'operation': 'updateNetworkSensorMqttBroker'
        }
        resource = f'/networks/{networkId}/sensor/mqttBrokers/{mqttBrokerId}'

        body_params = ['enabled', ]
        payload = {k.strip(): v for k, v in kwargs.items() if k.strip() in body_params}
        action = {
            "resource": resource,
            "operation": "update",
            "body": payload
        }
        return action
        






import urllib


class ActionBatchCellularGateway(object):
    def __init__(self):
        super(ActionBatchCellularGateway, self).__init__()
        


    def updateDeviceCellularGatewayLan(self, serial: str, **kwargs):
        """
        **Update the LAN Settings for a single MG.**
        https://developer.cisco.com/meraki/api-v1/#!update-device-cellular-gateway-lan

        - serial (string): Serial
        - reservedIpRanges (array): list of all reserved IP ranges for a single MG
        - fixedIpAssignments (array): list of all fixed IP assignments for a single MG
        """

        kwargs.update(locals())

        metadata = {
            'tags': ['cellularGateway', 'configure', 'lan'],
            'operation': 'updateDeviceCellularGatewayLan'
        }
        resource = f'/devices/{serial}/cellularGateway/lan'

        body_params = ['reservedIpRanges', 'fixedIpAssignments', ]
        payload = {k.strip(): v for k, v in kwargs.items() if k.strip() in body_params}
        action = {
            "resource": resource,
            "operation": "update",
            "body": payload
        }
        return action
        





    def updateDeviceCellularGatewayPortForwardingRules(self, serial: str, **kwargs):
        """
        **Updates the port forwarding rules for a single MG.**
        https://developer.cisco.com/meraki/api-v1/#!update-device-cellular-gateway-port-forwarding-rules

        - serial (string): Serial
        - rules (array): An array of port forwarding params
        """

        kwargs.update(locals())

        metadata = {
            'tags': ['cellularGateway', 'configure', 'portForwardingRules'],
            'operation': 'updateDeviceCellularGatewayPortForwardingRules'
        }
        resource = f'/devices/{serial}/cellularGateway/portForwardingRules'

        body_params = ['rules', ]
        payload = {k.strip(): v for k, v in kwargs.items() if k.strip() in body_params}
        action = {
            "resource": resource,
            "operation": "update",
            "body": payload
        }
        return action
        





    def updateNetworkCellularGatewayConnectivityMonitoringDestinations(self, networkId: str, **kwargs):
        """
        **Update the connectivity testing destinations for an MG network**
        https://developer.cisco.com/meraki/api-v1/#!update-network-cellular-gateway-connectivity-monitoring-destinations

        - networkId (string): Network ID
        - destinations (array): The list of connectivity monitoring destinations
        """

        kwargs.update(locals())

        metadata = {
            'tags': ['cellularGateway', 'configure', 'connectivityMonitoringDestinations'],
            'operation': 'updateNetworkCellularGatewayConnectivityMonitoringDestinations'
        }
        resource = f'/networks/{networkId}/cellularGateway/connectivityMonitoringDestinations'

        body_params = ['destinations', ]
        payload = {k.strip(): v for k, v in kwargs.items() if k.strip() in body_params}
        action = {
            "resource": resource,
            "operation": "update",
            "body": payload
        }
        return action
        





    def updateNetworkCellularGatewayDhcp(self, networkId: str, **kwargs):
        """
        **Update common DHCP settings of MGs**
        https://developer.cisco.com/meraki/api-v1/#!update-network-cellular-gateway-dhcp

        - networkId (string): Network ID
        - dhcpLeaseTime (string): DHCP Lease time for all MG of the network. Possible values are '30 minutes', '1 hour', '4 hours', '12 hours', '1 day' or '1 week'.
        - dnsNameservers (string): DNS name servers mode for all MG of the network. Possible values are: 'upstream_dns', 'google_dns', 'opendns', 'custom'.
        - dnsCustomNameservers (array): list of fixed IPs representing the the DNS Name servers when the mode is 'custom'
        """

        kwargs.update(locals())

        metadata = {
            'tags': ['cellularGateway', 'configure', 'dhcp'],
            'operation': 'updateNetworkCellularGatewayDhcp'
        }
        resource = f'/networks/{networkId}/cellularGateway/dhcp'

        body_params = ['dhcpLeaseTime', 'dnsNameservers', 'dnsCustomNameservers', ]
        payload = {k.strip(): v for k, v in kwargs.items() if k.strip() in body_params}
        action = {
            "resource": resource,
            "operation": "update",
            "body": payload
        }
        return action
        





    def updateNetworkCellularGatewaySubnetPool(self, networkId: str, **kwargs):
        """
        **Update the subnet pool and mask configuration for MGs in the network.**
        https://developer.cisco.com/meraki/api-v1/#!update-network-cellular-gateway-subnet-pool

        - networkId (string): Network ID
        - mask (integer): Mask used for the subnet of all MGs in  this network.
        - cidr (string): CIDR of the pool of subnets. Each MG in this network will automatically pick a subnet from this pool.
        """

        kwargs.update(locals())

        metadata = {
            'tags': ['cellularGateway', 'configure', 'subnetPool'],
            'operation': 'updateNetworkCellularGatewaySubnetPool'
        }
        resource = f'/networks/{networkId}/cellularGateway/subnetPool'

        body_params = ['mask', 'cidr', ]
        payload = {k.strip(): v for k, v in kwargs.items() if k.strip() in body_params}
        action = {
            "resource": resource,
            "operation": "update",
            "body": payload
        }
        return action
        





    def updateNetworkCellularGatewayUplink(self, networkId: str, **kwargs):
        """
        **Updates the uplink settings for your MG network.**
        https://developer.cisco.com/meraki/api-v1/#!update-network-cellular-gateway-uplink

        - networkId (string): Network ID
        - bandwidthLimits (object): The bandwidth settings for the 'cellular' uplink
        """

        kwargs.update(locals())

        metadata = {
            'tags': ['cellularGateway', 'configure', 'uplink'],
            'operation': 'updateNetworkCellularGatewayUplink'
        }
        resource = f'/networks/{networkId}/cellularGateway/uplink'

        body_params = ['bandwidthLimits', ]
        payload = {k.strip(): v for k, v in kwargs.items() if k.strip() in body_params}
        action = {
            "resource": resource,
            "operation": "update",
            "body": payload
        }
        return action
        





    def updateOrganizationCellularGatewayEsimsInventory(self, organizationId: str, id: str, **kwargs):
        """
        **Toggle the status of an eSIM**
        https://developer.cisco.com/meraki/api-v1/#!update-organization-cellular-gateway-esims-inventory

        - organizationId (string): Organization ID
        - id (string): ID
        - status (string): Status the eSIM will be updated to
        """

        kwargs.update(locals())

        metadata = {
            'tags': ['cellularGateway', 'configure', 'esims', 'inventory'],
            'operation': 'updateOrganizationCellularGatewayEsimsInventory'
        }
        resource = f'/organizations/{organizationId}/cellularGateway/esims/inventory/{id}'

        body_params = ['status', ]
        payload = {k.strip(): v for k, v in kwargs.items() if k.strip() in body_params}
        action = {
            "resource": resource,
            "operation": "update",
            "body": payload
        }
        return action
        





    def createOrganizationCellularGatewayEsimsServiceProvidersAccount(self, organizationId: str, accountId: str, apiKey: str, serviceProvider: dict, title: str, username: str):
        """
        **Add a service provider account.**
        https://developer.cisco.com/meraki/api-v1/#!create-organization-cellular-gateway-esims-service-providers-account

        - organizationId (string): Organization ID
        - accountId (string): Service provider account ID
        - apiKey (string): Service provider account API key
        - serviceProvider (object): Service Provider information
        - title (string): Service provider account name
        - username (string): Service provider account username
        """

        kwargs = locals()

        metadata = {
            'tags': ['cellularGateway', 'configure', 'esims', 'serviceProviders', 'accounts'],
            'operation': 'createOrganizationCellularGatewayEsimsServiceProvidersAccount'
        }
        resource = f'/organizations/{organizationId}/cellularGateway/esims/serviceProviders/accounts'

        body_params = ['accountId', 'apiKey', 'serviceProvider', 'title', 'username', ]
        payload = {k.strip(): v for k, v in kwargs.items() if k.strip() in body_params}
        action = {
            "resource": resource,
            "operation": "create",
            "body": payload
        }
        return action
        





    def updateOrganizationCellularGatewayEsimsServiceProvidersAccount(self, organizationId: str, accountId: str, **kwargs):
        """
        **Edit service provider account info stored in Meraki's database.**
        https://developer.cisco.com/meraki/api-v1/#!update-organization-cellular-gateway-esims-service-providers-account

        - organizationId (string): Organization ID
        - accountId (string): Account ID
        - title (string): Service provider account name used on the Meraki UI
        - apiKey (string): Service provider account API key
        """

        kwargs.update(locals())

        metadata = {
            'tags': ['cellularGateway', 'configure', 'esims', 'serviceProviders', 'accounts'],
            'operation': 'updateOrganizationCellularGatewayEsimsServiceProvidersAccount'
        }
        resource = f'/organizations/{organizationId}/cellularGateway/esims/serviceProviders/accounts/{accountId}'

        body_params = ['title', 'apiKey', ]
        payload = {k.strip(): v for k, v in kwargs.items() if k.strip() in body_params}
        action = {
            "resource": resource,
            "operation": "update",
            "body": payload
        }
        return action
        





    def deleteOrganizationCellularGatewayEsimsServiceProvidersAccount(self, organizationId: str, accountId: str):
        """
        **Remove a service provider account's integration with the Dashboard.**
        https://developer.cisco.com/meraki/api-v1/#!delete-organization-cellular-gateway-esims-service-providers-account

        - organizationId (string): Organization ID
        - accountId (string): Account ID
        """

        metadata = {
            'tags': ['cellularGateway', 'configure', 'esims', 'serviceProviders', 'accounts'],
            'operation': 'deleteOrganizationCellularGatewayEsimsServiceProvidersAccount'
        }
        resource = f'/organizations/{organizationId}/cellularGateway/esims/serviceProviders/accounts/{accountId}'

        action = {
            "resource": resource,
            "operation": "destroy",
        }
        return action
        





    def createOrganizationCellularGatewayEsimsSwap(self, organizationId: str, swaps: list):
        """
        **Swap which profile an eSIM uses.**
        https://developer.cisco.com/meraki/api-v1/#!create-organization-cellular-gateway-esims-swap

        - organizationId (string): Organization ID
        - swaps (array): Each object represents a swap for one eSIM
        """

        kwargs = locals()

        metadata = {
            'tags': ['cellularGateway', 'configure', 'esims', 'swap'],
            'operation': 'createOrganizationCellularGatewayEsimsSwap'
        }
        resource = f'/organizations/{organizationId}/cellularGateway/esims/swap'

        body_params = ['swaps', ]
        payload = {k.strip(): v for k, v in kwargs.items() if k.strip() in body_params}
        action = {
            "resource": resource,
            "operation": "swap",
            "body": payload
        }
        return action
        





    def updateOrganizationCellularGatewayEsimsSwap(self, id: str, organizationId: str):
        """
        **Get the status of a profile swap.**
        https://developer.cisco.com/meraki/api-v1/#!update-organization-cellular-gateway-esims-swap

        - id (string): eSIM EID
        - organizationId (string): Organization ID
        """

        metadata = {
            'tags': ['cellularGateway', 'configure', 'esims', 'swap'],
            'operation': 'updateOrganizationCellularGatewayEsimsSwap'
        }
        resource = f'/organizations/{organizationId}/cellularGateway/esims/swap/{id}'

        action = {
            "resource": resource,
            "operation": "status",
        }
        return action
        






import urllib


class CampusGateway(object):
    def __init__(self, session):
        super(CampusGateway, self).__init__()
        self._session = session
        


    def createNetworkCampusGatewayCluster(self, networkId: str, name: str, uplinks: list, tunnels: list, nameservers: dict, portChannels: list, **kwargs):
        """
        **Create a cluster and add campus gateways to it**
        https://developer.cisco.com/meraki/api-v1/#!create-network-campus-gateway-cluster

        - networkId (string): Network ID
        - name (string): Name of the new cluster
        - uplinks (array): Uplink interface settings of the cluster
        - tunnels (array): Tunnel interface settings of the cluster: Reuse uplink or specify tunnel interface
        - nameservers (object): Nameservers of the cluster
        - portChannels (array): Port channel settings of the cluster
        - devices (array): Devices to be added to the cluster
        - notes (string): Notes about cluster with max size of 511 characters allowed
        """

        kwargs.update(locals())

        metadata = {
            'tags': ['campusGateway', 'configure', 'clusters'],
            'operation': 'createNetworkCampusGatewayCluster'
        }
        networkId = urllib.parse.quote(str(networkId), safe='')
        resource = f'/networks/{networkId}/campusGateway/clusters'

        body_params = ['name', 'uplinks', 'tunnels', 'nameservers', 'portChannels', 'devices', 'notes', ]
        payload = {k.strip(): v for k, v in kwargs.items() if k.strip() in body_params}

        return self._session.post(metadata, resource, payload)
        


    def updateNetworkCampusGatewayCluster(self, networkId: str, clusterId: str, **kwargs):
        """
        **Update a cluster and add/remove campus gateways to/from it**
        https://developer.cisco.com/meraki/api-v1/#!update-network-campus-gateway-cluster

        - networkId (string): Network ID
        - clusterId (string): Cluster ID
        - name (string): Name of the cluster
        - uplinks (array): Uplink interface settings of the cluster
        - tunnels (array): Tunnel interface settings of the cluster: Reuse uplink or specify tunnel interface
        - nameservers (object): Nameservers of the cluster
        - portChannels (array): Port channel settings of the cluster
        - devices (array): Devices in the cluster
        - notes (string): Notes about cluster with max size of 511 characters allowed
        """

        kwargs.update(locals())

        metadata = {
            'tags': ['campusGateway', 'configure', 'clusters'],
            'operation': 'updateNetworkCampusGatewayCluster'
        }
        networkId = urllib.parse.quote(str(networkId), safe='')
        clusterId = urllib.parse.quote(str(clusterId), safe='')
        resource = f'/networks/{networkId}/campusGateway/clusters/{clusterId}'

        body_params = ['name', 'uplinks', 'tunnels', 'nameservers', 'portChannels', 'devices', 'notes', ]
        payload = {k.strip(): v for k, v in kwargs.items() if k.strip() in body_params}

        return self._session.put(metadata, resource, payload)
        


    def getOrganizationCampusGatewayDevicesUplinksLocalOverridesByDevice(self, organizationId: str, total_pages=1, direction='next', **kwargs):
        """
        **Uplink overrides configured locally on Campus Gateway devices in an organization.**
        https://developer.cisco.com/meraki/api-v1/#!get-organization-campus-gateway-devices-uplinks-local-overrides-by-device

        - organizationId (string): Organization ID
        - total_pages (integer or string): use with perPage to get total results up to total_pages*perPage; -1 or "all" for all pages
        - direction (string): direction to paginate, either "next" (default) or "prev" page
        - serials (array): A list of serial numbers. The returned devices will be filtered to only include these serials.
        - perPage (integer): The number of entries per page returned. Acceptable range is 3 - 1000. Default is 1000.
        - startingAfter (string): A token used by the server to indicate the start of the page. Often this is a timestamp or an ID but it is not limited to those. This parameter should not be defined by client applications. The link for the first, last, prev, or next page in the HTTP Link header should define it.
        - endingBefore (string): A token used by the server to indicate the end of the page. Often this is a timestamp or an ID but it is not limited to those. This parameter should not be defined by client applications. The link for the first, last, prev, or next page in the HTTP Link header should define it.
        """

        kwargs.update(locals())

        metadata = {
            'tags': ['campusGateway', 'configure', 'devices', 'uplinks', 'localOverrides', 'byDevice'],
            'operation': 'getOrganizationCampusGatewayDevicesUplinksLocalOverridesByDevice'
        }
        organizationId = urllib.parse.quote(str(organizationId), safe='')
        resource = f'/organizations/{organizationId}/campusGateway/devices/uplinks/localOverrides/byDevice'

        query_params = ['serials', 'perPage', 'startingAfter', 'endingBefore', ]
        params = {k.strip(): v for k, v in kwargs.items() if k.strip() in query_params}

        array_params = ['serials', ]
        for k, v in kwargs.items():
            if k.strip() in array_params:
                params[f'{k.strip()}[]'] = kwargs[f'{k}']
                params.pop(k.strip())

        return self._session.get_pages(metadata, resource, params, total_pages, direction)
        



import urllib


class ActionBatchSpaces(object):
    def __init__(self):
        super(ActionBatchSpaces, self).__init__()
        


    def removeOrganizationSpacesIntegration(self, organizationId: str):
        """
        **Remove the Spaces integration from Meraki**
        https://developer.cisco.com/meraki/api-v1/#!remove-organization-spaces-integration

        - organizationId (string): Organization ID
        """

        metadata = {
            'tags': ['organizations', 'configure', 'spaces', 'integration'],
            'operation': 'removeOrganizationSpacesIntegration'
        }
        resource = f'/organizations/{organizationId}/spaces/integration/remove'

        action = {
            "resource": resource,
            "operation": "integration",
        }
        return action
        






import urllib


class AsyncSpaces:
    def __init__(self, session):
        super().__init__()
        self._session = session
        


    def removeOrganizationSpacesIntegration(self, organizationId: str):
        """
        **Remove the Spaces integration from Meraki**
        https://developer.cisco.com/meraki/api-v1/#!remove-organization-spaces-integration

        - organizationId (string): Organization ID
        """

        metadata = {
            'tags': ['organizations', 'configure', 'spaces', 'integration'],
            'operation': 'removeOrganizationSpacesIntegration'
        }
        organizationId = urllib.parse.quote(str(organizationId), safe='')
        resource = f'/organizations/{organizationId}/spaces/integration/remove'

        return self._session.post(metadata, resource)
        
